"""
Test basics of Minidump debugging.
"""

import lldb
import os
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class MiniDumpUUIDTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def verify_module(self, module, verify_path, verify_uuid):
        # Compare the filename and the directory separately. We are avoiding
        # SBFileSpec.fullpath because it causes a slash/backslash confusion
        # on Windows.  Similarly, we compare the directories using normcase
        # because they may contain a Linux-style relative path from the
        # minidump appended to a Windows-style root path from the host.
        self.assertEqual(os.path.basename(verify_path), module.GetFileSpec().basename)
        self.assertEqual(
            os.path.normcase(os.path.dirname(verify_path)),
            os.path.normcase(module.GetFileSpec().dirname or ""),
        )
        self.assertEqual(verify_uuid, module.GetUUIDString())

    def get_minidump_modules(self, yaml_file, exe=None):
        minidump_path = self.getBuildArtifact(os.path.basename(yaml_file) + ".dmp")
        self.yaml2obj(yaml_file, minidump_path)
        self.target = self.dbg.CreateTarget(exe)
        self.process = self.target.LoadCore(minidump_path)
        return self.target.modules

    def test_zero_uuid_modules(self):
        """
        Test multiple modules having a MINIDUMP_MODULE.CvRecord that is valid,
        but contains a PDB70 value whose age is zero and whose UUID values are
        all zero. Prior to a fix all such modules would be duplicated to the
        first one since the UUIDs claimed to be valid and all zeroes. Now we
        ensure that the UUID is not valid for each module and that we have
        each of the modules in the target after loading the core
        """
        modules = self.get_minidump_modules("linux-arm-zero-uuids.yaml")
        self.assertEqual(2, len(modules))
        self.verify_module(modules[0], "/file/does/not/exist/a", None)
        self.verify_module(modules[1], "/file/does/not/exist/b", None)

    def test_uuid_modules_no_age(self):
        """
        Test multiple modules having a MINIDUMP_MODULE.CvRecord that is valid,
        and contains a PDB70 value whose age is zero and whose UUID values are
        valid. Ensure we decode the UUID and don't include the age field in the UUID.
        """
        modules = self.get_minidump_modules("linux-arm-uuids-no-age.yaml")
        modules = self.target.modules
        self.assertEqual(2, len(modules))
        self.verify_module(modules[0], "/tmp/a", "01020304-0506-0708-090A-0B0C0D0E0F10")
        self.verify_module(modules[1], "/tmp/b", "0A141E28-323C-4650-5A64-6E78828C96A0")

    def test_uuid_modules_no_age_apple(self):
        """
        Test multiple modules having a MINIDUMP_MODULE.CvRecord that is valid,
        and contains a PDB70 value whose age is zero and whose UUID values are
        valid. Ensure we decode the UUID and don't include the age field in the UUID.
        Also ensure that the first uint32_t is byte swapped, along with the next
        two uint16_t values. Breakpad incorrectly byte swaps these values when it
        saves Darwin minidump files.
        """
        modules = self.get_minidump_modules("macos-arm-uuids-no-age.yaml")
        modules = self.target.modules
        self.assertEqual(2, len(modules))
        self.verify_module(modules[0], "/tmp/a", "04030201-0605-0807-090A-0B0C0D0E0F10")
        self.verify_module(modules[1], "/tmp/b", "281E140A-3C32-5046-5A64-6E78828C96A0")

    def test_uuid_modules_with_age(self):
        """
        Test multiple modules having a MINIDUMP_MODULE.CvRecord that is valid,
        and contains a PDB70 value whose age is valid and whose UUID values are
        valid. Ensure we decode the UUID and include the age field in the UUID.
        """
        modules = self.get_minidump_modules("linux-arm-uuids-with-age.yaml")
        self.assertEqual(2, len(modules))
        self.verify_module(
            modules[0], "/tmp/a", "01020304-0506-0708-090A-0B0C0D0E0F10-10101010"
        )
        self.verify_module(
            modules[1], "/tmp/b", "0A141E28-323C-4650-5A64-6E78828C96A0-20202020"
        )

    def test_uuid_modules_elf_build_id_16(self):
        """
        Test multiple modules having a MINIDUMP_MODULE.CvRecord that is valid,
        and contains an ELF build ID whose value is valid and is 16 bytes long.
        """
        modules = self.get_minidump_modules("linux-arm-uuids-elf-build-id-16.yaml")
        self.assertEqual(2, len(modules))
        self.verify_module(modules[0], "/tmp/a", "01020304-0506-0708-090A-0B0C0D0E0F10")
        self.verify_module(modules[1], "/tmp/b", "0A141E28-323C-4650-5A64-6E78828C96A0")

    def test_uuid_modules_elf_build_id_20(self):
        """
        Test multiple modules having a MINIDUMP_MODULE.CvRecord that is valid,
        and contains an ELF build ID whose value is valid and is 20 bytes long.
        """
        modules = self.get_minidump_modules("linux-arm-uuids-elf-build-id-20.yaml")
        self.assertEqual(2, len(modules))
        self.verify_module(
            modules[0], "/tmp/a", "01020304-0506-0708-090A-0B0C0D0E0F10-11121314"
        )
        self.verify_module(
            modules[1], "/tmp/b", "0A141E28-323C-4650-5A64-6E78828C96A0-AAB4BEC8"
        )

    def test_uuid_modules_elf_build_id_zero(self):
        """
        Test multiple modules having a MINIDUMP_MODULE.CvRecord that is valid,
        and contains an ELF build ID whose value is all zero.
        """
        modules = self.get_minidump_modules("linux-arm-uuids-elf-build-id-zero.yaml")
        self.assertEqual(2, len(modules))
        self.verify_module(modules[0], "/not/exist/a", None)
        self.verify_module(modules[1], "/not/exist/b", None)

    def test_uuid_modules_elf_build_id_same(self):
        """
        Test multiple modules having a MINIDUMP_MODULE.CvRecord that is
        valid, and contains an ELF build ID whose value is the same. There
        is an assert in the PlaceholderObjectFile that was firing when we
        encountered this which was crashing the process that was checking
        if PlaceholderObjectFile.m_base was the same as the address this
        fake module was being loaded at. We need to ensure we don't crash
        in such cases and that we add both modules even though they have
        the same UUID.
        """
        modules = self.get_minidump_modules("linux-arm-same-uuids.yaml")
        self.assertEqual(2, len(modules))
        self.verify_module(
            modules[0],
            "/file/does/not/exist/a",
            "11223344-1122-3344-1122-334411223344-11223344",
        )
        self.verify_module(
            modules[1],
            "/file/does/not/exist/b",
            "11223344-1122-3344-1122-334411223344-11223344",
        )

    def test_partial_uuid_match(self):
        """
        Breakpad has been known to create minidump files using CvRecord in each
        module whose signature is set to PDB70 where the UUID only contains the
        first 16 bytes of a 20 byte ELF build ID. Code was added to
        ProcessMinidump.cpp to deal with this and allows partial UUID matching.

        This test verifies that if we have a minidump with a 16 byte UUID, that
        we are able to associate a symbol file with a 20 byte UUID only if the
        first 16 bytes match. In this case we will see the path from the file
        we found in the test directory and the 20 byte UUID from the actual
        file, not the 16 byte shortened UUID from the minidump.
        """
        so_path = self.getBuildArtifact("libuuidmatch.so")
        self.yaml2obj("libuuidmatch.yaml", so_path)
        cmd = 'settings set target.exec-search-paths "%s"' % (os.path.dirname(so_path))
        self.dbg.HandleCommand(cmd)
        modules = self.get_minidump_modules("linux-arm-partial-uuids-match.yaml")
        self.assertEqual(1, len(modules))
        self.verify_module(
            modules[0], so_path, "7295E17C-6668-9E05-CBB5-DEE5003865D5-5267C116"
        )

    def test_partial_uuid_mismatch(self):
        """
        Breakpad has been known to create minidump files using CvRecord in each
        module whose signature is set to PDB70 where the UUID only contains the
        first 16 bytes of a 20 byte ELF build ID. Code was added to
        ProcessMinidump.cpp to deal with this and allows partial UUID matching.

        This test verifies that if we have a minidump with a 16 byte UUID, that
        we are not able to associate a symbol file with a 20 byte UUID only if
        any of the first 16 bytes do not match. In this case we will see the UUID
        from the minidump file and the path from the minidump file.
        """
        so_path = self.getBuildArtifact("libuuidmismatch.so")
        self.yaml2obj("libuuidmismatch.yaml", so_path)
        cmd = 'settings set target.exec-search-paths "%s"' % (os.path.dirname(so_path))
        self.dbg.HandleCommand(cmd)
        modules = self.get_minidump_modules("linux-arm-partial-uuids-mismatch.yaml")
        self.assertEqual(1, len(modules))
        self.verify_module(
            modules[0],
            "/invalid/path/on/current/system/libuuidmismatch.so",
            "7295E17C-6668-9E05-CBB5-DEE5003865D5",
        )

    def test_breakpad_hash_match(self):
        """
        Breakpad creates minidump files using CvRecord in each module whose
        signature is set to PDB70 where the UUID is a hash generated by
        breakpad of the .text section. This is only done when the
        executable has no ELF build ID.

        This test verifies that if we have a minidump with a 16 byte UUID,
        that we are able to associate a symbol file with no ELF build ID
        and match it up by hashing the .text section.
        """
        so_path = self.getBuildArtifact("libbreakpad.so")
        self.yaml2obj("libbreakpad.yaml", so_path)
        cmd = 'settings set target.exec-search-paths "%s"' % (os.path.dirname(so_path))
        self.dbg.HandleCommand(cmd)
        modules = self.get_minidump_modules("linux-arm-breakpad-uuid-match.yaml")
        self.assertEqual(1, len(modules))
        # LLDB makes up it own UUID as well when there is no build ID so we
        # will check that this matches.
        self.verify_module(modules[0], so_path, "D9C480E8")

    def test_breakpad_hash_match_sysroot(self):
        """
        Check that we can match the breakpad .text section hash when the
        module is located under a user-provided sysroot.
        """
        sysroot_path = os.path.join(self.getBuildDir(), "mock_sysroot")
        # Create the directory under the sysroot where the minidump reports
        # the module.
        so_dir = os.path.join(
            sysroot_path, "invalid", "path", "on", "current", "system"
        )
        so_path = os.path.join(so_dir, "libbreakpad.so")
        lldbutil.mkdir_p(so_dir)
        self.yaml2obj("libbreakpad.yaml", so_path)
        self.runCmd("platform select remote-linux --sysroot '%s'" % sysroot_path)
        modules = self.get_minidump_modules("linux-arm-breakpad-uuid-match.yaml")
        self.assertEqual(1, len(modules))
        # LLDB makes up its own UUID as well when there is no build ID so we
        # will check that this matches.
        self.verify_module(modules[0], so_path, "D9C480E8")

    def test_breakpad_hash_match_sysroot_decoy(self):
        """
        Check that we can match the breakpad .text section hash when there is
        a module with the right name but wrong contents under a user-provided
        sysroot, and the right module is at the given search path..
        """
        sysroot_path = os.path.join(self.getBuildDir(), "mock_sysroot")
        # Create the directory under the sysroot where the minidump reports
        # the module.
        decoy_dir = os.path.join(
            sysroot_path, "invalid", "path", "on", "current", "system"
        )
        decoy_path = os.path.join(decoy_dir, "libbreakpad.so")
        lldbutil.mkdir_p(decoy_dir)
        self.yaml2obj("libbreakpad-decoy.yaml", decoy_path)
        self.runCmd("platform select remote-linux --sysroot '%s'" % sysroot_path)
        so_dir = os.path.join(self.getBuildDir(), "searchpath_dir")
        so_path = os.path.join(so_dir, "libbreakpad.so")
        lldbutil.mkdir_p(so_dir)
        self.yaml2obj("libbreakpad.yaml", so_path)
        self.runCmd('settings set target.exec-search-paths "%s"' % so_dir)
        modules = self.get_minidump_modules("linux-arm-breakpad-uuid-match.yaml")
        self.assertEqual(1, len(modules))
        # LLDB makes up its own UUID as well when there is no build ID so we
        # will check that this matches.
        self.verify_module(modules[0], so_path, "D9C480E8")

    def test_breakpad_overflow_hash_match(self):
        """
        This is a similar to test_breakpad_hash_match, but it verifies that
        if the .text section does not end on a 16 byte boundary, then it
        will overflow into the next section's data by up to 15 bytes. This
        verifies that we are able to match what breakpad does as it will do
        this.
        """
        so_path = self.getBuildArtifact("libbreakpad.so")
        self.yaml2obj("libbreakpad-overflow.yaml", so_path)
        cmd = 'settings set target.exec-search-paths "%s"' % (os.path.dirname(so_path))
        self.dbg.HandleCommand(cmd)
        modules = self.get_minidump_modules("linux-arm-breakpad-uuid-match.yaml")
        self.assertEqual(1, len(modules))
        # LLDB makes up it own UUID as well when there is no build ID so we
        # will check that this matches.
        self.verify_module(modules[0], so_path, "48EB9FD7")

    def test_breakpad_hash_match_exe_outside_sysroot(self):
        """
        Check that we can match the breakpad .text section hash when the
        module is specified as the exe during launch, and a syroot is
        provided, which does not contain the exe.
        """
        sysroot_path = os.path.join(self.getBuildDir(), "mock_sysroot")
        lldbutil.mkdir_p(sysroot_path)
        so_dir = os.path.join(self.getBuildDir(), "binary")
        so_path = os.path.join(so_dir, "libbreakpad.so")
        lldbutil.mkdir_p(so_dir)
        self.yaml2obj("libbreakpad.yaml", so_path)
        self.runCmd("platform select remote-linux --sysroot '%s'" % sysroot_path)
        modules = self.get_minidump_modules(
            "linux-arm-breakpad-uuid-match.yaml", so_path
        )
        self.assertEqual(1, len(modules))
        # LLDB makes up its own UUID as well when there is no build ID so we
        # will check that this matches.
        self.verify_module(modules[0], so_path, "D9C480E8")

    def test_facebook_hash_match(self):
        """
        Breakpad creates minidump files using CvRecord in each module whose
        signature is set to PDB70 where the UUID is a hash generated by
        breakpad of the .text section and Facebook modified this hash to
        avoid collisions. This is only done when the executable has no ELF
        build ID.

        This test verifies that if we have a minidump with a 16 byte UUID,
        that we are able to associate a symbol file with no ELF build ID
        and match it up by hashing the .text section like Facebook does.
        """
        so_path = self.getBuildArtifact("libbreakpad.so")
        self.yaml2obj("libbreakpad.yaml", so_path)
        cmd = 'settings set target.exec-search-paths "%s"' % (os.path.dirname(so_path))
        self.dbg.HandleCommand(cmd)
        modules = self.get_minidump_modules("linux-arm-facebook-uuid-match.yaml")
        self.assertEqual(1, len(modules))
        # LLDB makes up it own UUID as well when there is no build ID so we
        # will check that this matches.
        self.verify_module(modules[0], so_path, "D9C480E8")

    def test_relative_module_name(self):
        old_cwd = os.getcwd()
        self.addTearDownHook(lambda: os.chdir(old_cwd))
        os.chdir(self.getBuildDir())
        name = "file-with-a-name-unlikely-to-exist-in-the-current-directory.so"
        open(name, "a").close()
        modules = self.get_minidump_modules(
            self.getSourcePath("relative_module_name.yaml")
        )
        self.assertEqual(1, len(modules))
        self.verify_module(modules[0], name, None)

    def test_add_module_build_id_16(self):
        """
        Test that adding module with 16 byte UUID returns the existing
        module or fails.
        """
        modules = self.get_minidump_modules("linux-arm-uuids-elf-build-id-16.yaml")
        self.assertEqual(2, len(modules))

        # Add the existing modules.
        self.assertEqual(
            modules[0],
            self.target.AddModule(
                "/some/local/a", "", "01020304-0506-0708-090A-0B0C0D0E0F10"
            ),
        )
        self.assertEqual(
            modules[1],
            self.target.AddModule(
                "/some/local/b", "", "0A141E28-323C-4650-5A64-6E78828C96A0"
            ),
        )

        # Adding modules with non-existing UUID should fail.
        self.assertFalse(
            self.target.AddModule(
                "a", "", "12345678-1234-1234-1234-123456789ABC"
            ).IsValid()
        )
        self.assertFalse(
            self.target.AddModule(
                "a", "", "01020304-0506-0708-090A-0B0C0D0E0F10-12345678"
            ).IsValid()
        )

    def test_add_module_build_id_20(self):
        """
        Test that adding module with 20 byte UUID returns the existing
        module or fails.
        """
        modules = self.get_minidump_modules("linux-arm-uuids-elf-build-id-20.yaml")

        # Add the existing modules.
        self.assertEqual(
            modules[0],
            self.target.AddModule(
                "/some/local/a", "", "01020304-0506-0708-090A-0B0C0D0E0F10-11121314"
            ),
        )
        self.assertEqual(
            modules[1],
            self.target.AddModule(
                "/some/local/b", "", "0A141E28-323C-4650-5A64-6E78828C96A0-AAB4BEC8"
            ),
        )

        # Adding modules with non-existing UUID should fail.
        self.assertFalse(
            self.target.AddModule(
                "a", "", "01020304-0506-0708-090A-0B0C0D0E0F10"
            ).IsValid()
        )
        self.assertFalse(
            self.target.AddModule(
                "a", "", "01020304-0506-0708-090A-0B0C0D0E0F10-12345678"
            ).IsValid()
        )

    def test_add_module_build_id_4(self):
        """
        Test that adding module with 4 byte UUID returns the existing
        module or fails.
        """
        modules = self.get_minidump_modules("linux-arm-uuids-elf-build-id-4.yaml")

        # Add the existing modules.
        self.assertEqual(
            modules[0], self.target.AddModule("/some/local/a.so", "", "01020304")
        )
        self.assertEqual(
            modules[1], self.target.AddModule("/some/local/b.so", "", "0A141E28")
        )

        # Adding modules with non-existing UUID should fail.
        self.assertFalse(
            self.target.AddModule(
                "a", "", "01020304-0506-0708-090A-0B0C0D0E0F10"
            ).IsValid()
        )
        self.assertFalse(self.target.AddModule("a", "", "01020305").IsValid())

    def test_remove_placeholder_add_real_module(self):
        """
        Test that removing a placeholder module and adding back the real
        module succeeds.
        """
        so_path = self.getBuildArtifact("libuuidmatch.so")
        self.yaml2obj("libuuidmatch.yaml", so_path)
        modules = self.get_minidump_modules("linux-arm-uuids-match.yaml")

        uuid = "7295E17C-6668-9E05-CBB5-DEE5003865D5-5267C116"
        self.assertEqual(1, len(modules))
        self.verify_module(modules[0], "/target/path/libuuidmatch.so", uuid)

        self.target.RemoveModule(modules[0])
        new_module = self.target.AddModule(so_path, "", uuid)

        self.verify_module(new_module, so_path, uuid)
        self.assertEqual(new_module, self.target.modules[0])
        self.assertEqual(1, len(self.target.modules))



"""Test the SBSaveCoreOptions APIs."""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


class SBSaveCoreOptionsAPICase(TestBase):
    basic_minidump = "basic_minidump.yaml"
    basic_minidump_different_pid = "basic_minidump_different_pid.yaml"

    def get_process_from_yaml(self, yaml_file):
        minidump_path = self.getBuildArtifact(os.path.basename(yaml_file) + ".dmp")
        print("minidump_path: " + minidump_path)
        self.yaml2obj(yaml_file, minidump_path)
        self.assertTrue(
            os.path.exists(minidump_path), "yaml2obj did not emit a minidump file"
        )
        target = self.dbg.CreateTarget(None)
        process = target.LoadCore(minidump_path)
        self.assertTrue(process.IsValid(), "Process is not valid")
        return process

    def get_basic_process(self):
        return self.get_process_from_yaml(self.basic_minidump)

    def get_basic_process_different_pid(self):
        return self.get_process_from_yaml(self.basic_minidump_different_pid)

    def test_plugin_name_assignment(self):
        """Test assignment ensuring valid plugin names only."""
        options = lldb.SBSaveCoreOptions()
        error = options.SetPluginName(None)
        self.assertTrue(error.Success())
        self.assertEqual(options.GetPluginName(), None)
        error = options.SetPluginName("Not a real plugin")
        self.assertTrue(error.Fail())
        self.assertEqual(options.GetPluginName(), None)
        error = options.SetPluginName("minidump")
        self.assertTrue(error.Success())
        self.assertEqual(options.GetPluginName(), "minidump")
        error = options.SetPluginName("")
        self.assertTrue(error.Success())
        self.assertEqual(options.GetPluginName(), None)

    def test_default_corestyle_behavior(self):
        """Test that the default core style is unspecified."""
        options = lldb.SBSaveCoreOptions()
        self.assertEqual(options.GetStyle(), lldb.eSaveCoreUnspecified)

    def test_adding_and_removing_thread(self):
        """Test adding and removing a thread from save core options."""
        self.assertTrue(self.dbg)
        options = lldb.SBSaveCoreOptions()
        process = self.get_basic_process()
        self.assertTrue(process.IsValid(), "Process is not valid")
        thread = process.GetThreadAtIndex(0)
        error = options.AddThread(thread)
        self.assertTrue(error.Success(), error.GetCString())
        removed_success = options.RemoveThread(thread)
        self.assertTrue(removed_success)
        removed_success = options.RemoveThread(thread)
        self.assertFalse(removed_success)

    def test_adding_thread_different_process(self):
        """Test adding and removing a thread from save core options."""
        options = lldb.SBSaveCoreOptions()
        process = self.get_basic_process()
        process_2 = self.get_basic_process_different_pid()
        thread = process.GetThreadAtIndex(0)
        error = options.AddThread(thread)
        self.assertTrue(error.Success())
        thread_2 = process_2.GetThreadAtIndex(0)
        error = options.AddThread(thread_2)
        self.assertTrue(error.Fail())
        options.Clear()
        error = options.AddThread(thread_2)
        self.assertTrue(error.Success())
        options.SetProcess(process)
        error = options.AddThread(thread_2)
        self.assertTrue(error.Fail())
        error = options.AddThread(thread)
        self.assertTrue(error.Success())

    def test_removing_and_adding_insertion_order(self):
        """Test insertion order is maintained when removing and adding threads."""
        options = lldb.SBSaveCoreOptions()
        process = self.get_basic_process()
        threads = []
        for x in range(0, 3):
            thread = process.GetThreadAtIndex(x)
            threads.append(thread)
            error = options.AddThread(thread)
            self.assertTrue(error.Success())

        # Get the middle thread, remove it, and insert it back.
        middle_thread = threads[1]
        self.assertTrue(options.RemoveThread(middle_thread))
        thread_collection = options.GetThreadsToSave()
        self.assertTrue(thread_collection is not None)
        self.assertEqual(thread_collection.GetSize(), 2)
        error = options.AddThread(middle_thread)
        self.assertTrue(error.Success())
        thread_collection = options.GetThreadsToSave()
        self.assertEqual(thread_collection.GetSize(), 3)
        self.assertIn(middle_thread, thread_collection)
        self.assertEqual(
            thread_collection[0].GetThreadID(),
            thread_collection.GetThreadAtIndex(0).GetThreadID(),
        )
        self.assertEqual(
            thread_collection[1].GetThreadID(),
            thread_collection.GetThreadAtIndex(1).GetThreadID(),
        )
        self.assertEqual(
            thread_collection[-1].GetThreadID(),
            thread_collection.GetThreadAtIndex(
                thread_collection.GetSize() - 1
            ).GetThreadID(),
        )
        with self.assertRaises(IndexError):
            thread_collection[thread_collection.GetSize()]
        with self.assertRaises(TypeError):
            thread_collection["invalid"]

    def test_get_current_size_in_bytes(self):
        """
        Tests that ensures GetCurrentSizeInBytes properly returns an error without a process,
        and the readable regions with a process.
        """

        options = lldb.SBSaveCoreOptions()
        options.SetStyle(lldb.eSaveCoreCustomOnly)
        process = self.get_basic_process()
        memory_range = lldb.SBMemoryRegionInfo()

        # Add the memory range of 0x1000-0x1100
        process.GetMemoryRegionInfo(0x1000, memory_range)
        options.AddMemoryRegionToSave(memory_range)

        # Check that we fail when we have no process set
        # even though we added a memory region.
        error = lldb.SBError()
        total = options.GetCurrentSizeInBytes(error)
        self.assertTrue(error.Fail(), error.GetCString())

        # Check that we don't get an error now that we've added a process
        options.SetProcess(process)
        total = options.GetCurrentSizeInBytes(error)
        self.assertTrue(error.Success(), error.GetCString())

        # Validate the size returned is the same size as the single region we added.
        expected_size = memory_range.GetRegionEnd() - memory_range.GetRegionBase()
        self.assertEqual(total, expected_size)

    def test_get_total_in_bytes_missing_requirements(self):
        """
        Tests the matrix of error responses that GetCurrentSizeInBytes
        """

        options = lldb.SBSaveCoreOptions()

        # No process, no style returns an error.
        error = lldb.SBError()
        total = options.GetCurrentSizeInBytes(error)
        self.assertTrue(error.Fail(), error.GetCString())

        # No process returns an error
        options.SetStyle(lldb.eSaveCoreCustomOnly)
        total = options.GetCurrentSizeInBytes(error)
        self.assertTrue(error.Fail(), error.GetCString())

        options.Clear()

        # No style returns an error
        process = self.get_basic_process()
        options.SetProcess(process)
        total = options.GetCurrentSizeInBytes(error)
        self.assertTrue(error.Fail(), error.GetCString())

        # Options that result in no valid data returns an error.
        options.SetStyle(lldb.eSaveCoreCustomOnly)
        total = options.GetCurrentSizeInBytes(error)
        self.assertTrue(error.Fail(), error.GetCString())

    def test_get_memory_regions_to_save(self):
        """
        Tests the matrix of responses for GetMemoryRegionsToSave
        """

        options = lldb.SBSaveCoreOptions()

        # Not specifying plugin or process should return an empty list.
        memory_list = options.GetMemoryRegionsToSave()
        self.assertEqual(0, memory_list.GetSize())

        # No style returns an empty list
        process = self.get_basic_process()
        options.SetProcess(process)
        memory_list = options.GetMemoryRegionsToSave()
        self.assertEqual(0, memory_list.GetSize())
        options.Clear()

        # No Process returns an empty list
        options.SetStyle(lldb.eSaveCoreCustomOnly)
        memory_list = options.GetMemoryRegionsToSave()
        self.assertEqual(0, memory_list.GetSize())
        options.Clear()

        # Validate we get back the single region we populate
        options.SetStyle(lldb.eSaveCoreCustomOnly)
        process = self.get_basic_process()
        options.SetProcess(process)
        memory_range = lldb.SBMemoryRegionInfo()

        # Add the memory range of 0x1000-0x1100
        process.GetMemoryRegionInfo(0x1000, memory_range)
        options.AddMemoryRegionToSave(memory_range)
        memory_list = options.GetMemoryRegionsToSave()
        self.assertEqual(1, memory_list.GetSize())
        read_region = lldb.SBMemoryRegionInfo()
        memory_list.GetMemoryRegionAtIndex(0, read_region)

        # Permissions from Process getLLDBRegion aren't matching up with
        # the live process permissions, so we're just checking the range for now.
        self.assertEqual(memory_range.GetRegionBase(), read_region.GetRegionBase())
        self.assertEqual(memory_range.GetRegionEnd(), read_region.GetRegionEnd())



"""Test custom import command to import files by path."""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ImportTestCase(TestBase):
    @add_test_categories(["pyapi"])
    @no_debug_info_test
    def test_import_command(self):
        """Import some Python scripts by path and test them"""
        self.run_test()

    def run_test(self):
        """Import some Python scripts by path and test them."""

        # This is the function to remove the custom commands in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("command script delete foo2cmd", check=False)
            self.runCmd("command script delete foocmd", check=False)
            self.runCmd("command script delete foobarcmd", check=False)
            self.runCmd("command script delete barcmd", check=False)
            self.runCmd("command script delete barothercmd", check=False)
            self.runCmd("command script delete TPcommandA", check=False)
            self.runCmd("command script delete TPcommandB", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        self.runCmd("command script import ./foo/foo.py --allow-reload")
        self.runCmd("command script import ./foo/foo2.py --allow-reload")
        self.runCmd("command script import ./foo/bar/foobar.py --allow-reload")
        self.runCmd("command script import ./bar/bar.py --allow-reload")

        self.expect(
            "command script import ''",
            error=True,
            startstr="error: module importing failed: empty path",
        )
        self.expect(
            "command script import ./nosuchfile.py",
            error=True,
            startstr="error: module importing failed: invalid pathname './nosuchfile.py'",
        )
        self.expect(
            "command script import ./nosuchfolder/",
            error=True,
            startstr="error: module importing failed: invalid pathname './nosuchfolder/'",
        )
        self.expect("command script import ./foo/foo.py", error=False)
        self.runCmd("command script import --allow-reload ./thepackage")
        self.expect("TPcommandA", substrs=["hello world A"])
        self.expect("TPcommandB", substrs=["hello world B"])

        self.runCmd("script import dummymodule")
        self.expect("command script import ./dummymodule.py", error=False)
        self.expect(
            "command script import --allow-reload ./dummymodule.py", error=False
        )

        self.runCmd("command script add -f foo.foo_function foocmd")
        self.runCmd("command script add -f foobar.foo_function foobarcmd")
        self.runCmd("command script add -f bar.bar_function barcmd")
        self.expect("foocmd hello", substrs=["foo says", "hello"])
        self.expect("foo2cmd hello", substrs=["foo2 says", "hello"])
        self.expect("barcmd hello", substrs=["barutil says", "bar told me", "hello"])
        self.expect(
            "barothercmd hello", substrs=["barutil says", "bar told me", "hello"]
        )
        self.expect("foobarcmd hello", substrs=["foobar says", "hello"])



"""
Tests that bool types work
"""
import lldb
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class CPPBoolTestCase(TestBase):
    def test_with_run_command(self):
        """Test that bool types work in the expression parser"""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// breakpoint 1", lldb.SBFileSpec("main.cpp")
        )

        self.expect_expr(
            "bool second_bool = my_bool; second_bool",
            result_type="bool",
            result_value="false",
        )
        self.expect_expr("my_bool = true", result_type="bool", result_value="true")



"""
Test that variables with signed types display correctly.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class SignedTypesTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break inside main().
        self.source = "main.cpp"
        self.line = line_number(self.source, "// Set break point at this line.")

    @expectedFailureAll(
        oslist=["windows"], archs=["i[3-6]86", "x86_64"], bugnumber="llvm.org/pr24489"
    )
    def test(self):
        """Test that variables with signed types display correctly."""
        self.build()

        # Run in synchronous mode
        self.dbg.SetAsync(False)

        # Create a target by the debugger.
        target = self.dbg.CreateTarget(self.getBuildArtifact("a.out"))
        self.assertTrue(target, VALID_TARGET)

        lldbutil.run_break_set_by_file_and_line(
            self, self.source, self.line, num_expected_locations=1, loc_exact=True
        )

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())
        self.assertTrue(process, PROCESS_IS_VALID)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # The breakpoint should have a hit count of 1.
        lldbutil.check_breakpoint(self, bpno=1, expected_hit_count=1)

        # Execute the puts().
        self.runCmd("thread step-over")

        # Test that signed types display correctly.
        self.expect(
            "frame variable --show-types --no-args",
            VARIABLES_DISPLAYED_CORRECTLY,
            patterns=[
                r"\((signed char|char)\) the_signed_char = 'c'",
                r"\((short int|short)\) the_signed_short = 99",
            ],
            substrs=[
                "(int) the_signed_int = 99",
                "(long) the_signed_long = 99",
                "(long long) the_signed_long_long = 99",
            ],
        )



"""
Test lldb-dap setBreakpoints request
"""

from dap_server import Source
import shutil
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
import lldbdap_testcase
import os


class TestDAP_setBreakpoints(lldbdap_testcase.DAPTestCaseBase):
    SHARED_BUILD_TESTCASE = False

    def setUp(self):
        lldbdap_testcase.DAPTestCaseBase.setUp(self)

        self.main_basename = "main-copy.cpp"
        self.main_path = os.path.realpath(self.getBuildArtifact(self.main_basename))

    @skipIfWindows
    def test_source_map(self):
        """
        This test simulates building two files in a folder, and then moving
        each source to a different folder. Then, the debug session is started
        with the corresponding source maps to have breakpoints and frames
        working.
        """
        self.build_and_create_debug_adapter()

        other_basename = "other-copy.c"
        other_path = self.getBuildArtifact(other_basename)

        source_folder = os.path.dirname(self.main_path)

        new_main_folder = os.path.join(source_folder, "moved_main")
        new_other_folder = os.path.join(source_folder, "moved_other")

        new_main_path = os.path.join(new_main_folder, self.main_basename)
        new_other_path = os.path.join(new_other_folder, other_basename)

        # move the sources
        os.mkdir(new_main_folder)
        os.mkdir(new_other_folder)
        shutil.move(self.main_path, new_main_path)
        shutil.move(other_path, new_other_path)

        main_line = line_number("main.cpp", "break 12")
        other_line = line_number("other.c", "break other")

        program = self.getBuildArtifact("a.out")
        source_map = [
            [source_folder, new_main_folder],
            [source_folder, new_other_folder],
        ]
        self.launch(program, sourceMap=source_map)

        # breakpoint in main.cpp
        response = self.dap_server.request_setBreakpoints(
            Source.build(path=new_main_path), [main_line]
        )
        breakpoints = response["body"]["breakpoints"]
        self.assertEqual(len(breakpoints), 1)
        breakpoint = breakpoints[0]
        self.assertEqual(breakpoint["line"], main_line)
        self.assertTrue(breakpoint["verified"])
        self.assertEqual(self.main_basename, breakpoint["source"]["name"])
        self.assertEqual(new_main_path, breakpoint["source"]["path"])

        # 2nd breakpoint, which is from a dynamically loaded library
        response = self.dap_server.request_setBreakpoints(
            Source.build(path=new_other_path), [other_line]
        )
        breakpoints = response["body"]["breakpoints"]
        breakpoint = breakpoints[0]
        self.assertEqual(breakpoint["line"], other_line)
        self.assertFalse(breakpoint["verified"])
        self.assertEqual(other_basename, breakpoint["source"]["name"])
        self.assertEqual(new_other_path, breakpoint["source"]["path"])
        other_breakpoint_id = breakpoint["id"]

        self.dap_server.request_continue()
        self.verify_breakpoint_hit([other_breakpoint_id])

        # 2nd breakpoint again, which should be valid at this point
        response = self.dap_server.request_setBreakpoints(
            Source.build(path=new_other_path), [other_line]
        )
        breakpoints = response["body"]["breakpoints"]
        breakpoint = breakpoints[0]
        self.assertEqual(breakpoint["line"], other_line)
        self.assertTrue(breakpoint["verified"])
        self.assertEqual(other_basename, breakpoint["source"]["name"])
        self.assertEqual(new_other_path, breakpoint["source"]["path"])

        # now we check the stack trace making sure that we got mapped source paths
        frames = self.dap_server.request_stackTrace()["body"]["stackFrames"]

        self.assertEqual(frames[0]["source"]["name"], other_basename)
        self.assertEqual(frames[0]["source"]["path"], new_other_path)

        self.assertEqual(frames[1]["source"]["name"], self.main_basename)
        self.assertEqual(frames[1]["source"]["path"], new_main_path)

    @skipIfWindows
    def test_set_and_clear(self):
        """Tests setting and clearing source file and line breakpoints.
        This packet is a bit tricky on the debug adapter side since there
        is no "clearBreakpoints" packet. Source file and line breakpoints
        are set by sending a "setBreakpoints" packet with a source file
        specified and zero or more source lines. If breakpoints have been
        set in the source file before, any existing breakpoints must remain
        set, and any new breakpoints must be created, and any breakpoints
        that were in previous requests and are not in the current request
        must be removed. This function tests this setting and clearing
        and makes sure things happen correctly. It doesn't test hitting
        breakpoints and the functionality of each breakpoint, like
        'conditions' and 'hitCondition' settings."""
        first_line = line_number("main.cpp", "break 12")
        second_line = line_number("main.cpp", "break 13")
        third_line = line_number("main.cpp", "break 14")
        lines = [first_line, third_line, second_line]

        # Visual Studio Code Debug Adapters have no way to specify the file
        # without launching or attaching to a process, so we must start a
        # process in order to be able to set breakpoints.
        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program)

        # Set 3 breakpoints and verify that they got set correctly
        response = self.dap_server.request_setBreakpoints(
            Source.build(path=self.main_path), lines
        )
        line_to_id = {}
        breakpoints = response["body"]["breakpoints"]
        self.assertEqual(
            len(breakpoints),
            len(lines),
            "expect %u source breakpoints" % (len(lines)),
        )
        for index, breakpoint in enumerate(breakpoints):
            line = breakpoint["line"]
            self.assertEqual(line, lines[index])
            # Store the "id" of the breakpoint that was set for later
            line_to_id[line] = breakpoint["id"]
            self.assertTrue(breakpoint["verified"], "expect breakpoint verified")

        # There is no breakpoint delete packet, clients just send another
        # setBreakpoints packet with the same source file with fewer lines.
        # Below we remove the second line entry and call the setBreakpoints
        # function again. We want to verify that any breakpoints that were set
        # before still have the same "id". This means we didn't clear the
        # breakpoint and set it again at the same location. We also need to
        # verify that the second line location was actually removed.
        lines.remove(second_line)
        # Set 2 breakpoints and verify that the previous breakpoints that were
        # set above are still set.
        response = self.dap_server.request_setBreakpoints(
            Source.build(path=self.main_path), lines
        )
        breakpoints = response["body"]["breakpoints"]
        self.assertEqual(
            len(breakpoints),
            len(lines),
            "expect %u source breakpoints" % (len(lines)),
        )
        for index, breakpoint in enumerate(breakpoints):
            line = breakpoint["line"]
            self.assertEqual(line, lines[index])
            # Verify the same breakpoints are still set within LLDB by
            # making sure the breakpoint ID didn't change
            self.assertEqual(
                line_to_id[line],
                breakpoint["id"],
                "verify previous breakpoints stayed the same",
            )
            self.assertTrue(breakpoint["verified"], "expect breakpoint still verified")

        # Now get the full list of breakpoints set in the target and verify
        # we have only 2 breakpoints set. The response above could have told
        # us about 2 breakpoints, but we want to make sure we don't have the
        # third one still set in the target
        response = self.dap_server.request_testGetTargetBreakpoints()
        breakpoints = response["body"]["breakpoints"]
        self.assertEqual(
            len(breakpoints),
            len(lines),
            "expect %u source breakpoints" % (len(lines)),
        )
        for breakpoint in breakpoints:
            line = breakpoint["line"]
            # Verify the same breakpoints are still set within LLDB by
            # making sure the breakpoint ID didn't change
            self.assertEqual(
                line_to_id[line],
                breakpoint["id"],
                "verify previous breakpoints stayed the same",
            )
            self.assertIn(line, lines, "line expected in lines array")
            self.assertTrue(breakpoint["verified"], "expect breakpoint still verified")

        # Now clear all breakpoints for the source file by passing down an
        # empty lines array
        lines = []
        response = self.dap_server.request_setBreakpoints(
            Source.build(path=self.main_path), lines
        )
        breakpoints = response["body"]["breakpoints"]
        self.assertEqual(
            len(breakpoints),
            len(lines),
            "expect %u source breakpoints" % (len(lines)),
        )

        # Verify with the target that all breakpoints have been cleared
        response = self.dap_server.request_testGetTargetBreakpoints()
        breakpoints = response["body"]["breakpoints"]
        self.assertEqual(
            len(breakpoints),
            len(lines),
            "expect %u source breakpoints" % (len(lines)),
        )

        # Now set a breakpoint again in the same source file and verify it
        # was added.
        lines = [second_line]
        response = self.dap_server.request_setBreakpoints(
            Source.build(path=self.main_path), lines
        )
        if response:
            breakpoints = response["body"]["breakpoints"]
            self.assertEqual(
                len(breakpoints),
                len(lines),
                "expect %u source breakpoints" % (len(lines)),
            )
            for breakpoint in breakpoints:
                line = breakpoint["line"]
                self.assertIn(line, lines, "line expected in lines array")
                self.assertTrue(
                    breakpoint["verified"], "expect breakpoint still verified"
                )

        # Now get the full list of breakpoints set in the target and verify
        # we have only 2 breakpoints set. The response above could have told
        # us about 2 breakpoints, but we want to make sure we don't have the
        # third one still set in the target
        response = self.dap_server.request_testGetTargetBreakpoints()
        if response:
            breakpoints = response["body"]["breakpoints"]
            self.assertEqual(
                len(breakpoints),
                len(lines),
                "expect %u source breakpoints" % (len(lines)),
            )
            for breakpoint in breakpoints:
                line = breakpoint["line"]
                self.assertIn(line, lines, "line expected in lines array")
                self.assertTrue(
                    breakpoint["verified"], "expect breakpoint still verified"
                )

    @skipIfWindows
    def test_clear_breakpoints_unset_breakpoints(self):
        """Test clearing breakpoints like test_set_and_clear, but clear
        breakpoints by omitting the breakpoints array instead of sending an
        empty one."""
        lines = [
            line_number("main.cpp", "break 12"),
            line_number("main.cpp", "break 13"),
        ]

        # Visual Studio Code Debug Adapters have no way to specify the file
        # without launching or attaching to a process, so we must start a
        # process in order to be able to set breakpoints.
        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program)

        # Set one breakpoint and verify that it got set correctly.
        response = self.dap_server.request_setBreakpoints(
            Source.build(path=self.main_path), lines
        )
        line_to_id = {}
        breakpoints = response["body"]["breakpoints"]
        self.assertEqual(
            len(breakpoints), len(lines), "expect %u source breakpoints" % (len(lines))
        )
        for index, breakpoint in enumerate(breakpoints):
            line = breakpoint["line"]
            self.assertEqual(line, lines[index])
            # Store the "id" of the breakpoint that was set for later
            line_to_id[line] = breakpoint["id"]
            self.assertTrue(breakpoint["verified"], "expect breakpoint verified")

        # Now clear all breakpoints for the source file by not setting the
        # lines array.
        lines = None
        response = self.dap_server.request_setBreakpoints(
            Source.build(path=self.main_path), lines
        )
        breakpoints = response["body"]["breakpoints"]
        self.assertEqual(len(breakpoints), 0, "expect no source breakpoints")

        # Verify with the target that all breakpoints have been cleared.
        response = self.dap_server.request_testGetTargetBreakpoints()
        breakpoints = response["body"]["breakpoints"]
        self.assertEqual(len(breakpoints), 0, "expect no source breakpoints")

    @skipIfWindows
    def test_functionality(self):
        """Tests hitting breakpoints and the functionality of a single
        breakpoint, like 'conditions' and 'hitCondition' settings."""
        loop_line = line_number("main.cpp", "// break loop")

        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program)
        # Set a breakpoint at the loop line with no condition and no
        # hitCondition
        breakpoint_ids = self.set_source_breakpoints(self.main_path, [loop_line])
        self.assertEqual(len(breakpoint_ids), 1, "expect one breakpoint")
        self.dap_server.request_continue()

        # Verify we hit the breakpoint we just set
        self.verify_breakpoint_hit(breakpoint_ids)

        # Make sure i is zero at first breakpoint
        i = int(self.dap_server.get_local_variable_value("i"))
        self.assertEqual(i, 0, "i != 0 after hitting breakpoint")

        # Update the condition on our breakpoint
        new_breakpoint_ids = self.set_source_breakpoints(
            self.main_path, [loop_line], [{"condition": "i==4"}]
        )
        self.assertEqual(
            breakpoint_ids,
            new_breakpoint_ids,
            "existing breakpoint should have its condition " "updated",
        )

        self.continue_to_breakpoints(breakpoint_ids)
        i = int(self.dap_server.get_local_variable_value("i"))
        self.assertEqual(i, 4, "i != 4 showing conditional works")

        new_breakpoint_ids = self.set_source_breakpoints(
            self.main_path, [loop_line], [{"hitCondition": "2"}]
        )

        self.assertEqual(
            breakpoint_ids,
            new_breakpoint_ids,
            "existing breakpoint should have its condition " "updated",
        )

        # Continue with a hitCondition of 2 and expect it to skip 1 value
        self.continue_to_breakpoints(breakpoint_ids)
        i = int(self.dap_server.get_local_variable_value("i"))
        self.assertEqual(i, 6, "i != 6 showing hitCondition works")

        # continue after hitting our hitCondition and make sure it only goes
        # up by 1
        self.continue_to_breakpoints(breakpoint_ids)
        i = int(self.dap_server.get_local_variable_value("i"))
        self.assertEqual(i, 7, "i != 7 showing post hitCondition hits every time")

    @skipIfWindows
    def test_column_breakpoints(self):
        """Test setting multiple breakpoints in the same line at different columns."""
        loop_line = line_number("main.cpp", "// break loop")

        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program)

        # Set two breakpoints on the loop line at different columns.
        columns = [13, 39]
        response = self.dap_server.request_setBreakpoints(
            Source.build(path=self.main_path),
            [loop_line, loop_line],
            list({"column": c} for c in columns),
        )

        # Verify the breakpoints were set correctly
        breakpoints = response["body"]["breakpoints"]
        breakpoint_ids = []
        self.assertEqual(
            len(breakpoints),
            len(columns),
            "expect %u source breakpoints" % (len(columns)),
        )
        for index, breakpoint in enumerate(breakpoints):
            self.assertEqual(breakpoint["line"], loop_line)
            self.assertEqual(breakpoint["column"], columns[index])
            self.assertTrue(breakpoint["verified"], "expect breakpoint verified")
            breakpoint_ids.append(breakpoint["id"])

        # Continue to the first breakpoint,
        self.continue_to_breakpoints([breakpoint_ids[0]])

        # We should have stopped right before the call to `twelve`.
        # Step into and check we are inside `twelve`.
        self.stepIn()
        func_name = self.get_stackFrames()[0]["name"]
        self.assertEqual(func_name, "twelve(int)")

        # Continue to the second breakpoint.
        self.continue_to_breakpoints([breakpoint_ids[1]])

        # We should have stopped right before the call to `fourteen`.
        # Step into and check we are inside `fourteen`.
        self.stepIn()
        func_name = self.get_stackFrames()[0]["name"]
        self.assertEqual(func_name, "a::fourteen(int)")

    @skipIfWindows
    def test_hit_multiple_breakpoints(self):
        """Test that if we hit multiple breakpoints at the same address, they
        all appear in the stop reason."""
        breakpoint_lines = [
            line_number("main.cpp", "// break non-breakpointable line"),
            line_number("main.cpp", "// before loop"),
        ]

        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program)

        # Set a pair of breakpoints that will both resolve to the same address.
        breakpoint_ids = [
            int(bp_id)
            for bp_id in self.set_source_breakpoints(self.main_path, breakpoint_lines)
        ]
        self.assertEqual(len(breakpoint_ids), 2, "expected two breakpoints")
        self.dap_server.request_continue()
        print(breakpoint_ids)
        # Verify we hit both of the breakpoints we just set
        self.verify_all_breakpoints_hit(breakpoint_ids)



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @skipUnlessDarwin
    def test_objc_self(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// check self", lldb.SBFileSpec("main.m")
        )
        self.expect("frame variable _ivar", startstr="(int) _ivar = 30")

    @skipUnlessDarwin
    def test_objc_self_capture_idiom(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// check idiomatic self", lldb.SBFileSpec("main.m")
        )
        self.expect("frame variable weakSelf", startstr="(Classic *) weakSelf = 0x")
        self.expect("frame variable self", startstr="(Classic *) self = 0x")
        self.expect("frame variable _ivar", startstr="(int) _ivar = 30")



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *


class InvalidArgsExpressionTestCase(TestBase):
    @no_debug_info_test
    def test_invalid_lang(self):
        self.expect(
            "expression -l foo --",
            error=True,
            substrs=[
                "error: unknown language type: 'foo' for expression",
                "List of supported languages:",
                "c++",
                "c++11",
                "c++14",
            ],
        )

    @no_debug_info_test
    def test_invalid_all_thread(self):
        self.expect(
            "expression -a foo --",
            error=True,
            substrs=['error: invalid all-threads value setting: "foo"'],
        )

    @no_debug_info_test
    def test_invalid_ignore_br(self):
        self.expect(
            "expression -i foo --",
            error=True,
            substrs=['error: could not convert "foo" to a boolean value.'],
        )

    @no_debug_info_test
    def test_invalid_allow_jit(self):
        self.expect(
            "expression -j foo --",
            error=True,
            substrs=['error: could not convert "foo" to a boolean value.'],
        )

    @no_debug_info_test
    def test_invalid_timeout(self):
        self.expect(
            "expression -t foo --",
            error=True,
            substrs=['error: invalid timeout setting "foo"'],
        )

        self.expect(
            'expression -t "" --',
            error=True,
            substrs=['error: invalid timeout setting ""'],
        )

    @no_debug_info_test
    def test_invalid_unwind(self):
        self.expect(
            "expression -u foo --",
            error=True,
            substrs=['error: could not convert "foo" to a boolean value.'],
        )

    @no_debug_info_test
    def test_invalid_fixits(self):
        self.expect(
            "expression -X foo --",
            error=True,
            substrs=['error: could not convert "foo" to a boolean value.'],
        )



class LookupDictionary(dict):
    """
    a dictionary which can lookup value by key, or keys by value
    """

    def __init__(self, items=[]):
        """items can be a list of pair_lists or a dictionary"""
        dict.__init__(self, items)

    def get_keys_for_value(self, value, fail_value=None):
        """find the key(s) as a list given a value"""
        list_result = [item[0] for item in self.items() if item[1] == value]
        if len(list_result) > 0:
            return list_result
        return fail_value

    def get_first_key_for_value(self, value, fail_value=None):
        """return the first key of this dictionary given the value"""
        list_result = [item[0] for item in self.items() if item[1] == value]
        if len(list_result) > 0:
            return list_result[0]
        return fail_value

    def get_value(self, key, fail_value=None):
        """find the value given a key"""
        if key in self:
            return self[key]
        return fail_value


class Enum(LookupDictionary):
    def __init__(self, initial_value=0, items=[]):
        """items can be a list of pair_lists or a dictionary"""
        LookupDictionary.__init__(self, items)
        self.value = initial_value

    def set_value(self, v):
        v_typename = typeof(v).__name__
        if v_typename == "str":
            if str in self:
                v = self[v]
            else:
                v = 0
        else:
            self.value = v

    def get_enum_value(self):
        return self.value

    def get_enum_name(self):
        return self.__str__()

    def __str__(self):
        s = self.get_first_key_for_value(self.value, None)
        if s is None:
            s = "%#8.8x" % self.value
        return s

    def __repr__(self):
        return self.__str__()



"""
When a rebuild causes a location to be removed, make sure
we still handle the remaining locations correctly.
"""


import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import skipIfWindows
import os


class TestLocationsAfterRebuild(TestBase):
    SHARED_BUILD_TESTCASE = False
    # If your test case doesn't stress debug info, then
    # set this to true.  That way it won't be run once for
    # each debug info format.
    NO_DEBUG_INFO_TESTCASE = True

    # On Windows we cannot remove a file that lldb is debugging.
    @skipIfWindows
    def test_remaining_location_spec(self):
        """If we rebuild a couple of times some of the old locations
        get removed.  Make sure the command-line breakpoint id
        validator still works correctly."""
        self.build(dictionary={"C_SOURCES": "main.c", "EXE": "a.out"})

        path_to_exe = self.getBuildArtifact()

        (target, process, thread, bkpt) = lldbutil.run_to_name_breakpoint(self, "main")

        # Let the process continue to exit:
        process.Continue()
        self.assertEqual(process.GetState(), lldb.eStateExited, "Ran to completion")
        os.remove(path_to_exe)

        # We have to rebuild twice with changed sources to get
        # us to remove the first set of locations:
        self.build(dictionary={"C_SOURCES": "second_main.c", "EXE": "a.out"})

        (target, process, thread, bkpt) = lldbutil.run_to_breakpoint_do_run(
            self, target, bkpt
        )

        # Let the process continue to exit:
        process.Continue()
        self.assertEqual(process.GetState(), lldb.eStateExited, "Ran to completion")

        os.remove(path_to_exe)

        self.build(dictionary={"C_SOURCES": "third_main.c", "EXE": "a.out"})

        (target, process, thread, bkpt) = lldbutil.run_to_breakpoint_do_run(
            self, target, bkpt
        )

        # After enabling locate_module callback for main executables,
        # the number of locations may vary depending on the platform.
        num_locs = bkpt.GetNumLocations()
        bkpt_id = bkpt.GetID()

        self.assertGreater(
            num_locs,
            0,
            f"Expected at least one breakpoint location, but found {num_locs}",
        )

        # Iterate through all valid locations and verify we can disable each one.
        # This tests that breakpoint location IDs remain valid after rebuilds.
        for loc_idx in range(num_locs):
            loc = bkpt.GetLocationAtIndex(loc_idx)
            self.assertTrue(loc.IsValid(), f"Location at index {loc_idx} is not valid")

            # Get the actual location ID from the location object
            loc_id = loc.GetID()
            loc_string = f"{bkpt_id}.{loc_id}"
            self.runCmd(f"break disable {loc_string}")



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbgdbclient import GDBRemoteTestBase


class TestNoLocalFile(GDBRemoteTestBase):
    """Test the case where there is NO local copy of the file
    being debugged.  We shouldn't immediately error out, but
    rather lldb should ask debugserver if it knows about the file."""

    @skipIfXmlSupportMissing
    def test_with_python(self):
        self.do_test(False)

    @skipIfXmlSupportMissing
    def test_with_target_ceate(self):
        self.do_test(True)

    def do_test(self, use_target_create):
        self.absent_file = "/nosuch_dir/nosuch_subdir/nosuch_executable"
        self.a_packet_file = None

        class MyResponder(MockGDBServerResponder):
            def __init__(self, testcase):
                MockGDBServerResponder.__init__(self)
                self.after_launch = False
                self.testcase = testcase
                self.current_thread = 0

            def A(self, packet):
                # This is the main test, we want to see that lldb DID send the
                # A packet to get debugserver to load the file.
                # Skip the length and second length:
                print("Got A packet: {0}".format(packet))
                a_arr = packet.split(",")
                self.testcase.a_packet_file = bytearray.fromhex(a_arr[2]).decode()
                return "OK"

            def qXferRead(self, obj, annex, offset, length):
                if annex == "target.xml":
                    return (
                        """<?xml version="1.0"?>
                        <target version="1.0">
                          <architecture>i386:x86-64</architecture>
                          <feature name="org.gnu.gdb.i386.core">
                            <reg name="rip" bitsize="64" regnum="0" type="code_ptr" group="general"/>
                          </feature>
                        </target>""",
                        False,
                    )
                else:
                    return None, False

            def qC(self):
                if not self.after_launch:
                    return "QC0"
                return "0"

            def qfThreadInfo(self):
                if not self.after_launch:
                    return "OK"
                return "m0"

            def qsThreadInfo(self):
                if not self.after_launch:
                    return "OK"
                return "l"

            def qLaunchSuccess(self):
                return "OK"

            def qProcessInfo(self):
                return "$pid:10b70;parent-pid:10b20;real-uid:1f6;real-gid:14;effective-uid:1f6;effective-gid:14;cputype:1000007;cpusubtype:8;ptrsize:8;ostype:macosx;vendor:apple;endian:little;"

        error = lldb.SBError()
        self.server.responder = MyResponder(self)
        target = lldb.SBTarget()
        if use_target_create:
            create_cmd = "target create --arch x86_64-apple-macosx --platform remote-macosx --remote-file {0}".format(
                self.absent_file
            )
            self.runCmd(create_cmd)
            target = self.dbg.GetSelectedTarget()
            self.assertTrue(target.IsValid(), "Made a valid target")
        else:
            target = self.dbg.CreateTarget(
                None, "x86_64-apple-macosx", "remote-macosx", False, error
            )
            self.assertSuccess(error, "Made a valid target")

        launch_info = target.GetLaunchInfo()
        if not use_target_create:
            launch_info.SetExecutableFile(lldb.SBFileSpec(self.absent_file), True)
        flags = launch_info.GetLaunchFlags()
        flags |= lldb.eLaunchFlagStopAtEntry
        launch_info.SetLaunchFlags(flags)

        process = self.connect(target)
        self.assertTrue(process.IsValid(), "Process is valid")

        # We need to fetch the connected event:
        lldbutil.expect_state_changes(
            self, self.dbg.GetListener(), process, [lldb.eStateConnected]
        )

        self.server.responder.after_launch = True

        process = target.Launch(launch_info, error)

        self.assertSuccess(error, "Successfully launched.")
        self.assertState(
            process.GetState(), lldb.eStateStopped, "Should be stopped at entry"
        )
        self.assertIsNotNone(self.a_packet_file, "A packet was sent")
        self.assertEqual(
            self.absent_file, self.a_packet_file, "The A packet file was correct"
        )



import side_effect


def useless_function(first, second):
    print("I have the wrong number of arguments.")


def function(frame, bp_loc, dict):
    side_effect.bktptcmd = "function was here"


def another_function(frame, bp_loc, extra_args, dict):
    se_value = extra_args.GetValueForKey("side_effect")
    se_string = se_value.GetStringValue(100)
    side_effect.fancy = se_string


def a_third_function(frame, bp_loc, extra_args, dict):
    se_value = extra_args.GetValueForKey("side_effect")
    se_string = se_value.GetStringValue(100)
    side_effect.fancier = se_string


def a_list_function(frame, bp_loc, extra_args, dict):
    se_value = extra_args.GetValueForKey("side_effect")
    se_string = se_value.GetStringValue(100)
    side_effect.from_list = se_string


def empty_extra_args(frame, bp_loc, extra_args, dict):
    if extra_args.IsValid():
        side_effect.not_so_fancy = "Extra args should not be valid"
    side_effect.not_so_fancy = "Not so fancy"



"""
VTable pointers are signed with a discriminator that incorporates the object's
address (PointerAuthVTPtrAddressDiscrimination) and class type (
PointerAuthVTPtrTypeDiscrimination).
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestPtrAuthVTableExpressions(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipUnlessArm64eSupported
    def test_virtual_call_on_debuggee_object(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp", False)
        )

        self.expect_expr("d.value()", result_type="int", result_value="20")
        self.expect_expr("od.value()", result_type="int", result_value="30")

    @skipUnlessArm64eSupported
    def test_virtual_call_through_base_pointer(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp", False)
        )

        self.expect_expr("base_ptr->value()", result_type="int", result_value="20")

    @skipUnlessArm64eSupported
    def test_virtual_call_via_helper(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp", False)
        )

        self.expect_expr("call_value(&d)", result_type="int", result_value="20")
        self.expect_expr("call_value(&od)", result_type="int", result_value="30")
        self.expect_expr("call_value(base_ptr)", result_type="int", result_value="20")



"""
Test lldb's ability to read and write the AArch64 TLS registers.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class AArch64LinuxTLSRegisters(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setup(self, registers):
        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self,
            "main.c",
            line_number("main.c", "// Set break point at this line."),
            num_expected_locations=1,
        )

        lldbutil.run_break_set_by_file_and_line(
            self,
            "main.c",
            line_number("main.c", "// Set break point 2 at this line."),
            num_expected_locations=1,
        )

        if "tpidr2" in registers:
            self.runCmd("settings set target.run-args 1")
        self.runCmd("run", RUN_SUCCEEDED)

        if self.process().GetState() == lldb.eStateExited:
            self.fail("Test program failed to run.")

        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

    def check_registers(self, registers, values):
        regs = self.thread().GetSelectedFrame().GetRegisters()
        tls_regs = regs.GetFirstValueByName("Thread Local Storage Registers")
        self.assertTrue(tls_regs.IsValid(), "No TLS registers found.")

        for register in registers:
            tls_reg = tls_regs.GetChildMemberWithName(register)
            self.assertTrue(
                tls_reg.IsValid(), "{} register not found.".format(register)
            )
            self.assertEqual(tls_reg.GetValueAsUnsigned(), values[register])
            if register == "tpidr":
                self.expect("reg read tp", substrs=[hex(values[register])])

    def check_tls_reg(self, registers):
        self.setup(registers)

        # Since we can't predict what the value will be, the program has set
        # a target value for us to find.
        initial_values = {
            "tpidr": 0x1122334455667788,
            "tpidr2": 0x8877665544332211,
        }

        self.check_registers(registers, initial_values)

        # Their values should be restored if an expression modifies them.
        self.runCmd("expression expr_func()")

        self.check_registers(registers, initial_values)

        set_values = {
            "tpidr": 0x1111222233334444,
            "tpidr2": 0x4444333322221111,
        }

        # Set our own value(s) for the program to find.
        for register in registers:
            self.expect(
                "register write {} 0x{:x}".format(register, set_values[register])
            )

        self.expect("continue")

        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        for register in registers:
            self.expect("p {}_was_set".format(register), substrs=["true"])

    @skipUnlessArch("aarch64")
    @skipUnlessPlatform(["linux"])
    def test_tls_no_sme(self):
        if self.isAArch64SME():
            self.skipTest("SME must not be present.")

        self.check_tls_reg(["tpidr"])

    @skipUnlessArch("aarch64")
    @skipUnlessPlatform(["linux"])
    def test_tls_sme(self):
        if not self.isAArch64SME():
            self.skipTest("SME must be present.")

        self.check_tls_reg(["tpidr", "tpidr2"])

    @skipUnlessArch("aarch64")
    @skipUnlessPlatform(["linux"])
    def test_tpidr2_no_sme(self):
        if self.isAArch64SME():
            self.skipTest("SME must not be present.")

        self.setup("tpidr")

        regs = self.thread().GetSelectedFrame().GetRegisters()
        tls_regs = regs.GetFirstValueByName("Thread Local Storage Registers")
        self.assertTrue(tls_regs.IsValid(), "No TLS registers found.")
        tls_reg = tls_regs.GetChildMemberWithName("tpidr2")
        self.assertFalse(tls_reg.IsValid(), "tpdir2 should not be present without SME")



from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
from lldbsuite.test_event.build_exception import BuildError


class TestStepUntilAPI(TestBase):
    NO_DEBUG_INFO_TESTCASE = True
    SHARED_BUILD_TESTCASE = False

    def setUp(self):
        super().setUp()

        self.main_source = "main.c"
        self.main_spec = lldb.SBFileSpec(self.main_source)
        self.less_than_two = line_number("main.c", "Less than 2")
        self.greater_than_two = line_number("main.c", "Greater than or equal to 2.")
        self.back_out_in_main = line_number("main.c", "Back out in main")
        self.in_foo = line_number("main.c", "In foo")

    def _build_dict_for_discontinuity(self):
        return dict(
            CFLAGS_EXTRAS="-funique-basic-block-section-names "
            + "-ffunction-sections -fbasic-block-sections=list="
            + self.getSourcePath("function.list"),
            LD_EXTRAS="-Wl,--script=" + self.getSourcePath("symbol.order"),
        )

    def _do_until(self, build_dict, args, until_line, expected_line):
        self.build(dictionary=build_dict)
        launch_info = lldb.SBLaunchInfo(args)
        _, _, thread, _ = lldbutil.run_to_source_breakpoint(
            self, "At the start", self.main_spec, launch_info
        )

        self.assertSuccess(
            thread.StepOverUntil(self.frame(), self.main_spec, until_line)
        )

        self.runCmd("process status")

        line = self.frame().GetLineEntry().GetLine()
        self.assertEqual(
            line, expected_line, "Did not get the expected stop line number"
        )

    def _assertDiscontinuity(self):
        target = self.target()
        foo = target.FindFunctions("foo")
        self.assertEqual(len(foo), 1)
        foo = foo[0]

        call_me = self.target().FindFunctions("call_me")
        self.assertEqual(len(call_me), 1)
        call_me = call_me[0]

        foo_addr = foo.function.GetStartAddress().GetLoadAddress(target)
        found_before = False
        found_after = False
        for range in call_me.function.GetRanges():
            addr = range.GetBaseAddress().GetLoadAddress(target)
            if addr < foo_addr:
                found_before = True
            if addr > foo_addr:
                found_after = True

        self.assertTrue(
            found_before and found_after,
            "'foo' is not between 'call_me'" + str(foo) + str(call_me),
        )

    def test_hitting(self):
        """Test SBThread.StepOverUntil - targeting a line and hitting it."""
        self._do_until(None, None, self.less_than_two, self.less_than_two)

    @skipIf(oslist=lldbplatformutil.getDarwinOSTriples() + ["windows"])
    @skipIf(archs=no_match(["x86_64", "aarch64"]))
    @skipIf(compiler=no_match(["clang"]))
    def test_hitting_discontinuous(self):
        """Test SBThread.StepOverUntil - targeting a line and hitting it -- with
        discontinuous functions"""
        try:
            self._do_until(
                self._build_dict_for_discontinuity(),
                None,
                self.less_than_two,
                self.less_than_two,
            )
        except BuildError as ex:
            self.skipTest(f"failed to build with linker script.")

        self._assertDiscontinuity()

    def test_missing(self):
        """Test SBThread.StepOverUntil - targeting a line and missing it by stepping out to call site"""
        self._do_until(
            None, ["foo", "bar", "baz"], self.less_than_two, self.back_out_in_main
        )

    @skipIf(oslist=lldbplatformutil.getDarwinOSTriples() + ["windows"])
    @skipIf(archs=no_match(["x86_64", "aarch64"]))
    @skipIf(compiler=no_match(["clang"]))
    def test_missing_discontinuous(self):
        """Test SBThread.StepOverUntil - targeting a line and missing it by
        stepping out to call site -- with discontinuous functions"""
        try:
            self._do_until(
                self._build_dict_for_discontinuity(),
                ["foo", "bar", "baz"],
                self.less_than_two,
                self.back_out_in_main,
            )
        except BuildError as ex:
            self.skipTest(f"failed to build with linker script.")

        self._assertDiscontinuity()

    def test_bad_line(self):
        """Test that we get an error if attempting to step outside the current
        function"""
        self.build()
        _, _, thread, _ = lldbutil.run_to_source_breakpoint(
            self, "At the start", self.main_spec
        )
        self.assertIn(
            "step until target not in current function",
            thread.StepOverUntil(
                self.frame(), self.main_spec, self.in_foo
            ).GetCString(),
        )

    @skipIf(oslist=lldbplatformutil.getDarwinOSTriples() + ["windows"])
    @skipIf(archs=no_match(["x86_64", "aarch64"]))
    @skipIf(compiler=no_match(["clang"]))
    def test_bad_line_discontinuous(self):
        """Test that we get an error if attempting to step outside the current
        function -- and the function is discontinuous"""

        try:
            self.build(dictionary=self._build_dict_for_discontinuity())
            _, _, thread, _ = lldbutil.run_to_source_breakpoint(
                self, "At the start", self.main_spec
            )
        except BuildError as ex:
            self.skipTest(f"failed to build with linker script.")

        self.assertIn(
            "step until target not in current function",
            thread.StepOverUntil(
                self.frame(), self.main_spec, self.in_foo
            ).GetCString(),
        )
        self._assertDiscontinuity()



# RUN: %PYTHON %s | FileCheck %s
# REQUIRES: python-ge-311
# UNSUPPORTED: python-stable-abi
import gc
from contextlib import contextmanager

from mlir.ir import *
from mlir.dialects._ods_common import _cext
from mlir.dialects import arith, _arith_ops_gen


def run(f):
    print("\nTEST:", f.__name__)
    f()
    gc.collect()
    assert Context._get_live_count() == 0


# CHECK-LABEL: TEST: testInferLocations
@run
def testInferLocations():
    with Context() as ctx, loc_tracebacks():
        ctx.allow_unregistered_dialects = True

        op = Operation.create("custom.op1")
        one = arith.constant(IndexType.get(), 1)
        _cext.globals.register_traceback_file_exclusion(arith.__file__)
        two = arith.constant(IndexType.get(), 2)

        # fmt: off
        # CHECK: loc(callsite("testInferLocations"("{{.*}}[[SEP:[/\\]+]]test[[SEP]]python[[SEP]]ir[[SEP]]auto_location.py":{{[0-9]+}}:13 to :43) at callsite("run"("{{.*}}[[SEP]]test[[SEP]]python[[SEP]]ir[[SEP]]auto_location.py":14:4 to :7) at "<module>"("{{.*}}[[SEP]]test[[SEP]]python[[SEP]]ir[[SEP]]auto_location.py":{{[0-9]+}}:1 to :4))))
        # fmt: on
        print(op.location)

        # Test nesting of loc_tracebacks().
        with loc_tracebacks():
            # fmt: off
            # CHECK: loc(callsite("ConstantOp.__init__"("{{.*}}[[SEP]]mlir[[SEP]]dialects[[SEP]]arith.py":45:12 to :76) at callsite("constant"("{{.*}}[[SEP]]mlir[[SEP]]dialects[[SEP]]arith.py":90:40 to :81) at callsite("testInferLocations"("{{.*}}[[SEP]]test[[SEP]]python[[SEP]]ir[[SEP]]auto_location.py":{{[0-9]+}}:14 to :48) at callsite("run"("{{.*}}[[SEP]]test[[SEP]]python[[SEP]]ir[[SEP]]auto_location.py":{{[0-9]+}}:4 to :7) at "<module>"("{{.*}}[[SEP]]test[[SEP]]python[[SEP]]ir[[SEP]]auto_location.py":{{[0-9]+}}:1 to :4))))))
            # fmt: on
            print(one.location)

        # fmt: off
        # CHECK: loc(callsite("testInferLocations"("{{.*}}[[SEP]]test[[SEP]]python[[SEP]]ir[[SEP]]auto_location.py":{{[0-9]+}}:14 to :48) at callsite("run"("{{.*}}[[SEP]]test[[SEP]]python[[SEP]]ir[[SEP]]auto_location.py":{{[0-9]+}}:4 to :7) at "<module>"("{{.*}}[[SEP]]test[[SEP]]python[[SEP]]ir[[SEP]]auto_location.py":{{[0-9]+}}:1 to :4))))
        # fmt: on
        print(two.location)

        _cext.globals.register_traceback_file_inclusion(_arith_ops_gen.__file__)
        three = arith.constant(IndexType.get(), 3)
        # fmt: off
        # CHECK: loc(callsite("ConstantOp.__init__"("{{.*}}[[SEP]]mlir[[SEP]]dialects[[SEP]]_arith_ops_gen.py":{{[0-9]+}}:4 to :235) at callsite("testInferLocations"("{{.*}}[[SEP]]test[[SEP]]python[[SEP]]ir[[SEP]]auto_location.py":{{[0-9]+}}:16 to :50) at callsite("run"("{{.*}}[[SEP]]test[[SEP]]python[[SEP]]ir[[SEP]]auto_location.py":{{[0-9]+}}:4 to :7) at "<module>"("{{.*}}[[SEP]]test[[SEP]]python[[SEP]]ir[[SEP]]auto_location.py":{{[0-9]+}}:1 to :4)))))
        # fmt: on
        print(three.location)

        def foo():
            four = arith.constant(IndexType.get(), 4)
            print(four.location)

        # fmt: off
        # CHECK: loc(callsite("ConstantOp.__init__"("{{.*}}[[SEP]]mlir[[SEP]]dialects[[SEP]]_arith_ops_gen.py":{{[0-9]+}}:4 to :235) at callsite("testInferLocations.<locals>.foo"("{{.*}}[[SEP]]test[[SEP]]python[[SEP]]ir[[SEP]]auto_location.py":{{[0-9]+}}:19 to :53) at callsite("testInferLocations"("{{.*}}[[SEP]]test[[SEP]]python[[SEP]]ir[[SEP]]auto_location.py":{{[0-9]+}}:8 to :13) at callsite("run"("{{.*}}[[SEP]]test[[SEP]]python[[SEP]]ir[[SEP]]auto_location.py":{{[0-9]+}}:4 to :7) at "<module>"("{{.*}}[[SEP]]test[[SEP]]python[[SEP]]ir[[SEP]]auto_location.py":{{[0-9]+}}:1 to :4))))))
        # fmt: on
        foo()

        _cext.globals.register_traceback_file_exclusion(__file__)

        # fmt: off
        # CHECK: loc("ConstantOp.__init__"("{{.*}}[[SEP]]mlir[[SEP]]dialects[[SEP]]_arith_ops_gen.py":{{[0-9]+}}:4 to :235))
        # fmt: on
        foo()

        def bar1():
            def bar2():
                def bar3():
                    five = arith.constant(IndexType.get(), 5)
                    print(five.location)

                bar3()

            bar2()

        _cext.globals.register_traceback_file_inclusion(__file__)
        _cext.globals.register_traceback_file_exclusion(_arith_ops_gen.__file__)

        _cext.globals.set_loc_tracebacks_frame_limit(2)
        # fmt: off
        # CHECK: loc(callsite("testInferLocations.<locals>.bar1.<locals>.bar2.<locals>.bar3"("{{.*}}[[SEP]]test[[SEP]]python[[SEP]]ir[[SEP]]auto_location.py":{{[0-9]+}}:27 to :61) at "testInferLocations.<locals>.bar1.<locals>.bar2"("{{.*}}[[SEP]]test[[SEP]]python[[SEP]]ir[[SEP]]auto_location.py":{{[0-9]+}}:16 to :22)))
        # fmt: on
        bar1()

        _cext.globals.set_loc_tracebacks_frame_limit(1)
        # fmt: off
        # CHECK: loc("testInferLocations.<locals>.bar1.<locals>.bar2.<locals>.bar3"("{{.*}}[[SEP]]test[[SEP]]python[[SEP]]ir[[SEP]]auto_location.py":{{[0-9]+}}:27 to :61))
        # fmt: on
        bar1()

        _cext.globals.set_loc_tracebacks_frame_limit(0)
        # CHECK: loc(unknown)
        bar1()



#!/usr/bin/env python

import lldb
import shlex


def dump_module_sources(module, result):
    if module:
        print("Module: %s" % (module.file), file=result)
        for compile_unit in module.compile_units:
            if compile_unit.file:
                print("  %s" % (compile_unit.file), file=result)


def info_sources(debugger, command, result, dict):
    description = """This command will dump all compile units in any modules that are listed as arguments, or for all modules if no arguments are supplied."""
    module_names = shlex.split(command)
    target = debugger.GetSelectedTarget()
    if module_names:
        for module_name in module_names:
            dump_module_sources(target.module[module_name], result)
    else:
        for module in target.modules:
            dump_module_sources(module, result)


def __lldb_init_module(debugger, dict):
    # Add any commands contained in this module to LLDB
    debugger.HandleCommand("command script add -o -f sources.info_sources info_sources")
    print(
        'The "info_sources" command has been installed, type "help info_sources" or "info_sources --help" for detailed help.'
    )



"""Test that we properly vend children for a single entry NSDictionary"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ObjCSingleEntryDictionaryTestCase(TestBase):
    SHARED_BUILD_TESTCASE = False

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break inside main().
        self.line = line_number("main.m", "// break here")

    @skipUnlessDarwin
    @expectedFailureAll(
        oslist=["watchos"], bugnumber="rdar://problem/34642736"
    )  # bug in NSDictionary formatting on watchos
    def test_single_entry_dict(self):
        self.build()
        self.run_tests()

    @skipUnlessDarwin
    @expectedFailureAll(
        oslist=["watchos"], bugnumber="rdar://problem/34642736"
    )  # bug in NSDictionary formatting on watchos
    def test_single_entry_dict_no_const(self):
        disable_constant_classes = {
            "CFLAGS_EXTRAS": "-fno-constant-nsnumber-literals "
            + "-fno-constant-nsarray-literals "
            + "-fno-constant-nsdictionary-literals",
        }
        # FIXME: Remove compiler when flags are available upstream.
        self.build(dictionary=disable_constant_classes, compiler="xcrun clang")
        self.run_tests()

    def run_tests(self):
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Break inside the foo function which takes a bar_ptr argument.
        lldbutil.run_break_set_by_file_and_line(
            self, "main.m", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # The breakpoint should have a hit count of 1.
        lldbutil.check_breakpoint(self, bpno=1, expected_hit_count=1)

        d1 = self.frame().FindVariable("d1")
        d1.SetPreferSyntheticValue(True)
        d1.SetPreferDynamicValue(lldb.eDynamicCanRunTarget)

        self.assertEqual(d1.GetNumChildren(), 1, "dictionary has != 1 child elements")
        pair = d1.GetChildAtIndex(0)
        pair.SetPreferSyntheticValue(True)
        pair.SetPreferDynamicValue(lldb.eDynamicCanRunTarget)

        self.assertEqual(pair.GetNumChildren(), 2, "pair has != 2 child elements")

        key = pair.GetChildMemberWithName("key")
        value = pair.GetChildMemberWithName("value")

        key.SetPreferSyntheticValue(True)
        key.SetPreferDynamicValue(lldb.eDynamicCanRunTarget)
        value.SetPreferSyntheticValue(True)
        value.SetPreferDynamicValue(lldb.eDynamicCanRunTarget)

        self.assertEqual(key.GetSummary(), '@"key"', "key doesn't contain key")
        self.assertEqual(value.GetSummary(), '@"value"', "value doesn't contain value")



"""
Test lldb-dap launch request.
"""

import lldbdap_testcase


class TestDAP_launch_termination(lldbdap_testcase.DAPTestCaseBase):
    """
    Tests the correct termination of lldb-dap upon a 'disconnect' request.
    """

    def test(self):
        self.create_debug_adapter()
        # The underlying lldb-dap process must be alive
        self.assertEqual(self.dap_server.process.poll(), None)

        # The lldb-dap process should finish even though
        # we didn't close the communication socket explicitly
        self.dap_server.request_disconnect()

        # Wait until the underlying lldb-dap process dies.
        self.dap_server.process.wait(timeout=self.DEFAULT_TIMEOUT)

        # Check the return code
        self.assertEqual(self.dap_server.process.poll(), 0)



"""
Test the SBModule and SBTarget type lookup APIs.
"""

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TypeFindFirstTestCase(TestBase):
    def test_find_first_type(self):
        """
        Test SBTarget::FindFirstType() and SBModule::FindFirstType() APIs.

        This function had regressed after some past modification of the type
        lookup internal code where if we had multiple types with the same
        basename, FindFirstType() could end up failing depending on which
        type was found first in the debug info indexes. This test will
        ensure this doesn't regress in the future.

        The test also looks for a type defined in a different compilation unit
        to verify that SymbolFileDWARFDebugMap searches each symbol file in a
        module.
        """
        self.build()
        target = self.createTestTarget()
        exe_module = target.GetModuleAtIndex(0)
        self.assertTrue(exe_module.IsValid())
        # Test the SBTarget and SBModule APIs for FindFirstType
        for api in [target, exe_module]:
            integer_type = api.FindFirstType("Integer::Point")
            self.assertTrue(integer_type.IsValid())
            float_type = api.FindFirstType("Float::Point")
            self.assertTrue(float_type.IsValid())
            external_type = api.FindFirstType("OtherCompilationUnit::Type")
            self.assertTrue(external_type.IsValid())
            nonexistent_type = api.FindFirstType("NonexistentType")
            self.assertFalse(nonexistent_type.IsValid())



"""
Test SBLaunchInfo
"""


from lldbsuite.test.lldbtest import *


def lookup(info, key):
    for i in range(info.GetNumEnvironmentEntries()):
        KeyEqValue = info.GetEnvironmentEntryAtIndex(i)
        Key, Value = KeyEqValue.split("=")
        if Key == key:
            return Value
    return ""


class TestSBLaunchInfo(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_environment_getset(self):
        info = lldb.SBLaunchInfo(None)
        info.SetEnvironmentEntries(["FOO=BAR"], False)
        self.assertEqual(1, info.GetNumEnvironmentEntries())
        info.SetEnvironmentEntries(["BAR=BAZ"], True)
        self.assertEqual(2, info.GetNumEnvironmentEntries())
        self.assertEqual("BAR", lookup(info, "FOO"))
        self.assertEqual("BAZ", lookup(info, "BAR"))



import json
import re

import gdbremote_testcase
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


@skipIfMTE  # MTE security transition shims restrict socket operations.
class TestGdbRemoteThreadsInStopReply(gdbremote_testcase.GdbRemoteTestCaseBase):
    ENABLE_THREADS_IN_STOP_REPLY_ENTRIES = [
        "read packet: $QListThreadsInStopReply#21",
        "send packet: $OK#00",
    ]

    def gather_stop_reply_fields(self, thread_count, field_names):
        context, threads = self.launch_with_threads(thread_count)
        key_vals_text = context.get("stop_reply_kv")
        self.assertIsNotNone(key_vals_text)

        self.reset_test_sequence()
        self.add_register_info_collection_packets()
        self.add_process_info_collection_packets()

        context = self.expect_gdbremote_sequence()
        self.assertIsNotNone(context)
        hw_info = self.parse_hw_info(context)

        # Parse the stop reply contents.
        kv_dict = self.parse_key_val_dict(key_vals_text)

        result = dict()
        result["pc_register"] = hw_info["pc_register"]
        result["little_endian"] = hw_info["little_endian"]
        for key_field in field_names:
            result[key_field] = kv_dict.get(key_field)

        return result

    def gather_stop_reply_threads(self, thread_count):
        # Pull out threads from stop response.
        stop_reply_threads_text = self.gather_stop_reply_fields(
            thread_count, ["threads"]
        )["threads"]
        if stop_reply_threads_text:
            return [
                int(thread_id, 16) for thread_id in stop_reply_threads_text.split(",")
            ]
        else:
            return []

    def gather_stop_reply_pcs(self, thread_count):
        results = self.gather_stop_reply_fields(thread_count, ["threads", "thread-pcs"])
        if not results:
            return []

        threads_text = results["threads"]
        pcs_text = results["thread-pcs"]
        thread_ids = threads_text.split(",")
        pcs = pcs_text.split(",")
        self.assertEqual(len(thread_ids), len(pcs))

        thread_pcs = dict()
        for i in range(0, len(pcs)):
            thread_pcs[int(thread_ids[i], 16)] = pcs[i]

        result = dict()
        result["thread_pcs"] = thread_pcs
        result["pc_register"] = results["pc_register"]
        result["little_endian"] = results["little_endian"]
        return result

    def switch_endian(self, egg):
        return "".join(reversed(re.findall("..", egg)))

    def parse_hw_info(self, context):
        self.assertIsNotNone(context)
        process_info = self.parse_process_info_response(context)
        endian = process_info.get("endian")
        reg_info = self.parse_register_info_packets(context)
        (pc_lldb_reg_index, pc_reg_info) = self.find_pc_reg_info(reg_info)

        hw_info = dict()
        hw_info["pc_register"] = pc_lldb_reg_index
        hw_info["little_endian"] = endian == "little"
        return hw_info

    def gather_threads_info_pcs(self, pc_register, little_endian):
        self.reset_test_sequence()
        self.test_sequence.add_log_lines(
            [
                "read packet: $jThreadsInfo#c1",
                {
                    "direction": "send",
                    "regex": r"^\$(.*)#[0-9a-fA-F]{2}$",
                    "capture": {1: "threads_info"},
                },
            ],
            True,
        )

        context = self.expect_gdbremote_sequence()
        self.assertIsNotNone(context)
        threads_info = context.get("threads_info")
        register = str(pc_register)
        # The jThreadsInfo response is not valid JSON data, so we have to
        # clean it up first.
        jthreads_info = json.loads(re.sub(r"}]", "}", threads_info))
        thread_pcs = dict()
        for thread_info in jthreads_info:
            tid = thread_info["tid"]
            pc = thread_info["registers"][register]
            thread_pcs[tid] = self.switch_endian(pc) if little_endian else pc

        return thread_pcs

    def test_QListThreadsInStopReply_supported(self):
        self.build()
        self.set_inferior_startup_launch()
        procs = self.prep_debug_monitor_and_inferior()
        self.test_sequence.add_log_lines(
            self.ENABLE_THREADS_IN_STOP_REPLY_ENTRIES, True
        )

        context = self.expect_gdbremote_sequence()
        self.assertIsNotNone(context)

    @skipIfNetBSD
    @expectedFailureAll(oslist=["windows"])  # Extra threads present
    def test_stop_reply_reports_multiple_threads(self):
        self.build()
        self.set_inferior_startup_launch()
        # Gather threads from stop notification when QThreadsInStopReply is
        # enabled.
        self.test_sequence.add_log_lines(
            self.ENABLE_THREADS_IN_STOP_REPLY_ENTRIES, True
        )
        stop_reply_threads = self.gather_stop_reply_threads(5)
        self.assertEqual(len(stop_reply_threads), 5)

    @skipIfNetBSD
    def test_no_QListThreadsInStopReply_supplies_no_threads(self):
        self.build()
        self.set_inferior_startup_launch()
        # Gather threads from stop notification when QThreadsInStopReply is not
        # enabled.
        stop_reply_threads = self.gather_stop_reply_threads(5)
        self.assertEqual(len(stop_reply_threads), 0)

    @skipIfNetBSD
    def test_stop_reply_reports_correct_threads(self):
        self.build()
        self.set_inferior_startup_launch()
        # Gather threads from stop notification when QThreadsInStopReply is
        # enabled.
        thread_count = 5
        self.test_sequence.add_log_lines(
            self.ENABLE_THREADS_IN_STOP_REPLY_ENTRIES, True
        )
        stop_reply_threads = self.gather_stop_reply_threads(thread_count)

        # Gather threads from q{f,s}ThreadInfo.
        self.reset_test_sequence()
        self.add_threadinfo_collection_packets()

        context = self.expect_gdbremote_sequence()
        self.assertIsNotNone(context)

        threads = self.parse_threadinfo_packets(context)
        self.assertIsNotNone(threads)
        self.assertGreaterEqual(len(threads), thread_count)

        # Ensure each thread in q{f,s}ThreadInfo appears in stop reply threads
        for tid in threads:
            self.assertIn(tid, stop_reply_threads)

    @skipIfNetBSD
    @skipIfWindows  # Flaky on Windows
    def test_stop_reply_contains_thread_pcs(self):
        self.build()
        self.set_inferior_startup_launch()
        thread_count = 5
        self.test_sequence.add_log_lines(
            self.ENABLE_THREADS_IN_STOP_REPLY_ENTRIES, True
        )
        results = self.gather_stop_reply_pcs(thread_count)
        stop_reply_pcs = results["thread_pcs"]
        pc_register = results["pc_register"]
        little_endian = results["little_endian"]
        self.assertGreaterEqual(len(stop_reply_pcs), thread_count)

        threads_info_pcs = self.gather_threads_info_pcs(pc_register, little_endian)

        self.assertEqual(len(threads_info_pcs), len(stop_reply_pcs))
        for thread_id in stop_reply_pcs:
            self.assertIn(thread_id, threads_info_pcs)
            self.assertEqual(
                int(stop_reply_pcs[thread_id], 16), int(threads_info_pcs[thread_id], 16)
            )



"""Test that forward declarations don't cause bogus conflicts in namespaced types"""


import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil


class NamespaceDefinitionsTestCase(TestBase):
    # See also llvm.org/pr28948
    @expectedFailureAll(bugnumber="llvm.org/pr50814", compiler="gcc")
    @expectedFailureAll(
        bugnumber="llvm.org/pr28948",
        oslist=["linux"],
        compiler="gcc",
        archs=["arm$", "aarch64"],
    )
    @expectedFailureAll(oslist=["windows"])
    def test_expr(self):
        self.build()
        self.common_setup()

        self.expect(
            "expression -- Foo::MyClass()",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["thing = "],
        )

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break inside main().
        self.source = "main.cpp"
        self.line = line_number(self.source, "// Set breakpoint here")
        self.shlib_names = ["a", "b"]

    def common_setup(self):
        # Run in synchronous mode
        self.dbg.SetAsync(False)

        # Create a target by the debugger.
        target = self.dbg.CreateTarget(self.getBuildArtifact("a.out"))
        self.assertTrue(target, VALID_TARGET)

        # Break inside the foo function which takes a bar_ptr argument.
        lldbutil.run_break_set_by_file_and_line(
            self, self.source, self.line, num_expected_locations=1, loc_exact=True
        )

        # Register our shared libraries for remote targets so they get
        # automatically uploaded
        environment = self.registerSharedLibrariesWithTarget(target, self.shlib_names)

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(
            None, environment, self.get_process_working_directory()
        )
        self.assertTrue(process, PROCESS_IS_VALID)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # The breakpoint should have a hit count of 1.
        lldbutil.check_breakpoint(self, bpno=1, expected_hit_count=1)



# RUN: %PYTHON %s | FileCheck %s

import numpy as np
from mlir.ir import *
from mlir.dialects import quant


def run(f):
    print("\nTEST:", f.__name__)
    f()
    return f


# CHECK-LABEL: TEST: test_type_hierarchy
@run
def test_type_hierarchy():
    with Context():
        i8 = IntegerType.get_signless(8)
        any = Type.parse("!quant.any<i8<-8:7>:f32>")
        uniform = Type.parse("!quant.uniform<i8<-8:7>:f32, 0.99872:127>")
        per_axis = Type.parse("!quant.uniform<i8:f32:1, {2.0e+2,0.99872:120}>")
        sub_channel = Type.parse(
            "!quant.uniform<i8:f32:{0:1, 1:2}, {{2.0:10, 3.0:20}, {4.0:30, 5.0:40}}>"
        )
        calibrated = Type.parse("!quant.calibrated<f32<-0.998:1.2321>>")

        assert not isinstance(i8, quant.QuantizedType)
        assert isinstance(any, quant.QuantizedType)
        assert isinstance(uniform, quant.QuantizedType)
        assert isinstance(per_axis, quant.QuantizedType)
        assert isinstance(sub_channel, quant.QuantizedType)
        assert isinstance(calibrated, quant.QuantizedType)

        assert isinstance(any, quant.AnyQuantizedType)
        assert isinstance(uniform, quant.UniformQuantizedType)
        assert isinstance(per_axis, quant.UniformQuantizedPerAxisType)
        assert isinstance(sub_channel, quant.UniformQuantizedSubChannelType)
        assert isinstance(calibrated, quant.CalibratedQuantizedType)

        assert not isinstance(uniform, quant.AnyQuantizedType)
        assert not isinstance(per_axis, quant.UniformQuantizedType)
        assert not isinstance(sub_channel, quant.UniformQuantizedType)
        assert not isinstance(sub_channel, quant.UniformQuantizedPerAxisType)

        generic_t = Type(uniform)
        assert type(generic_t) is Type
        r = quant.QuantizedType.cast_to_storage_type(generic_t)
        assert isinstance(r, IntegerType)


# CHECK-LABEL: TEST: test_any_quantized_type
@run
def test_any_quantized_type():
    with Context():
        i8 = IntegerType.get_signless(8)
        f32 = F32Type.get()
        any = quant.AnyQuantizedType.get(
            quant.QuantizedType.FLAG_SIGNED, i8, f32, -8, 7
        )

        # CHECK: flags: 1
        print(f"flags: {any.flags}")
        # CHECK: signed: True
        print(f"signed: {any.is_signed}")
        # CHECK: storage type: i8
        print(f"storage type: {any.storage_type}")
        # CHECK: expressed type: f32
        print(f"expressed type: {any.expressed_type}")
        # CHECK: storage min: -8
        print(f"storage min: {any.storage_type_min}")
        # CHECK: storage max: 7
        print(f"storage max: {any.storage_type_max}")
        # CHECK: storage width: 8
        print(f"storage width: {any.storage_type_integral_width}")
        # CHECK: quantized element type: !quant.any<i8<-8:7>:f32>
        print(f"quantized element type: {any.quantized_element_type}")
        # CHECK: !quant.any<i8<-8:7>:f32>
        print(any)
        assert any == Type.parse("!quant.any<i8<-8:7>:f32>")


# CHECK-LABEL: TEST: test_uniform_type
@run
def test_uniform_type():
    with Context():
        i8 = IntegerType.get_signless(8)
        f32 = F32Type.get()
        uniform = quant.UniformQuantizedType.get(
            quant.UniformQuantizedType.FLAG_SIGNED, i8, f32, 0.99872, 127, -8, 7
        )

        # CHECK: scale: 0.99872
        print(f"scale: {uniform.scale}")
        # CHECK: zero point: 127
        print(f"zero point: {uniform.zero_point}")
        # CHECK: fixed point: False
        print(f"fixed point: {uniform.is_fixed_point}")
        # CHECK: !quant.uniform<i8<-8:7>:f32, 9.987200e-01:127>
        print(uniform)
        assert uniform == Type.parse("!quant.uniform<i8<-8:7>:f32, 0.99872:127>")


# CHECK-LABEL: TEST: test_uniform_per_axis_type
@run
def test_uniform_per_axis_type():
    with Context():
        i8 = IntegerType.get_signless(8)
        f32 = F32Type.get()
        per_axis = quant.UniformQuantizedPerAxisType.get(
            quant.QuantizedType.FLAG_SIGNED,
            i8,
            f32,
            [200, 0.99872],
            [0, 120],
            quantized_dimension=1,
            storage_type_min=quant.QuantizedType.default_minimum_for_integer(
                is_signed=True, integral_width=8
            ),
            storage_type_max=quant.QuantizedType.default_maximum_for_integer(
                is_signed=True, integral_width=8
            ),
        )

        # CHECK: scales: [200.0, 0.99872]
        print(f"scales: {per_axis.scales}")
        # CHECK: zero_points: [0, 120]
        print(f"zero_points: {per_axis.zero_points}")
        # CHECK: quantized dim: 1
        print(f"quantized dim: {per_axis.quantized_dimension}")
        # CHECK: fixed point: False
        print(f"fixed point: {per_axis.is_fixed_point}")
        # CHECK: !quant.uniform<i8:f32:1, {2.000000e+02,9.987200e-01:120}>
        print(per_axis)
        assert per_axis == Type.parse("!quant.uniform<i8:f32:1, {2.0e+2,0.99872:120}>")


# CHECK-LABEL: TEST: test_uniform_sub_channel_type
@run
def test_uniform_sub_channel_type():
    with Context():
        i8 = IntegerType.get_signless(8)
        f32 = F32Type.get()
        sub_channel = quant.UniformQuantizedSubChannelType.get(
            quant.QuantizedType.FLAG_SIGNED,
            i8,
            f32,
            DenseElementsAttr.get(
                np.asarray([2.0, 3.0, 4.0, 5.0], np.float32).reshape(2, 2)
            ),
            DenseElementsAttr.get(np.asarray([10, 20, 30, 40], np.int8).reshape(2, 2)),
            [0, 1],
            [1, 2],
            storage_type_min=quant.QuantizedType.default_minimum_for_integer(
                is_signed=True, integral_width=8
            ),
            storage_type_max=quant.QuantizedType.default_maximum_for_integer(
                is_signed=True, integral_width=8
            ),
        )

        # CHECK: quantized dimensions: [0, 1]
        print(f"quantized dimensions: {sub_channel.quantized_dimensions}")
        # CHECK: block sizes: [1, 2]
        print(f"block sizes: {sub_channel.block_sizes}")
        # CHECK: scales: {{\[}}[2. 3.]
        # CHECK:               [4. 5.]]
        print(f"scales: {np.asarray(sub_channel.scales)}")
        # CHECK: zero-points: {{\[}}[10 20]
        # CHECK:                    [30 40]]
        print(f"zero-points: {np.asarray(sub_channel.zero_points)}")
        # CHECK: !quant.uniform<i8:f32:{0:1, 1:2}, {{\{}}{2.000000e+00:10, 3.000000e+00:20}, {4.000000e+00:30, 5.000000e+00:40}}>
        print(sub_channel)
        assert sub_channel == Type.parse(
            "!quant.uniform<i8:f32:{0:1, 1:2},{{2.0:10, 3.0:20}, {4.0:30, 5.0:40}}>"
        )


# CHECK-LABEL: TEST: test_calibrated_type
@run
def test_calibrated_type():
    with Context():
        f32 = F32Type.get()
        calibrated = quant.CalibratedQuantizedType.get(f32, -0.998, 1.2321)

        # CHECK: min: -0.998
        print(f"min: {calibrated.min}")
        # CHECK: max: 1.2321
        print(f"max: {calibrated.max}")
        # CHECK: !quant.calibrated<f32<-0.998:1.232100e+00>>
        print(calibrated)
        assert calibrated == Type.parse("!quant.calibrated<f32<-0.998:1.2321>>")



"""
Test that backticks without a target should work (not infinite looping).
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class BackticksWithNoTargetTestCase(TestBase):
    @no_debug_info_test
    def test_backticks_no_target(self):
        """A simple test of backticks without a target."""
        self.expect("expression `1+2-3`", substrs=[" = 0"])



import lldb
import json
from intelpt_testcase import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
from lldbsuite.test.decorators import *


@skipIfNoIntelPT
class TestTraceStartStopMultipleThreads(TraceIntelPTTestCaseBase):
    @skipIf(oslist=no_match(["linux"]), archs=no_match(["i386", "x86_64"]))
    @testSBAPIAndCommands
    def testStartMultipleLiveThreads(self):
        self.build()
        exe = self.getBuildArtifact("a.out")

        self.dbg.CreateTarget(exe)

        self.expect("b main")
        self.expect("b 6")
        self.expect("b 11")

        self.expect("r")
        self.traceStartProcess()

        self.expect("continue")
        self.expect("thread trace dump instructions", substrs=["main.cpp:9"])

        # We'll see here the second thread
        self.expect("continue")
        self.expect("thread trace dump instructions", substrs=["main.cpp:4"])

        self.traceStopProcess()

    @skipIf(oslist=no_match(["linux"]), archs=no_match(["i386", "x86_64"]))
    @testSBAPIAndCommands
    def testStartMultipleLiveThreadsWithStops(self):
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.dbg.CreateTarget(exe)

        self.expect("b main")
        self.expect("b 6")
        self.expect("b 11")

        self.expect("r")

        self.traceStartProcess()

        # We'll see here the first thread
        self.expect("continue")

        # We are in thread 2
        self.expect("thread trace dump instructions", substrs=["main.cpp:9"])
        self.expect("thread trace dump instructions 2", substrs=["main.cpp:9"])

        # We stop tracing all
        self.expect("thread trace stop all")

        # The trace is still in memory
        self.expect("thread trace dump instructions 2", substrs=["main.cpp:9"])

        # We'll stop at the next breakpoint in thread 3, thread 2 and 3 will be alive, but only 3 traced.
        self.expect("continue")
        self.expect("thread trace dump instructions", substrs=["main.cpp:4"])
        self.expect("thread trace dump instructions 3", substrs=["main.cpp:4"])
        self.expect(
            "thread trace dump instructions 1", substrs=["not traced"], error=True
        )
        self.expect(
            "thread trace dump instructions 2", substrs=["not traced"], error=True
        )

        self.traceStopProcess()

    @skipIf(oslist=no_match(["linux"]), archs=no_match(["i386", "x86_64"]))
    def testStartMultipleLiveThreadsWithThreadStartAll(self):
        self.build()
        exe = self.getBuildArtifact("a.out")
        target = self.dbg.CreateTarget(exe)

        self.expect("b main")
        self.expect("b 6")
        self.expect("b 11")

        self.expect("r")

        self.expect("continue")
        # We are in thread 2
        self.expect("thread trace start all")
        # Now we have instructions in thread's 2 trace
        self.expect("n")

        self.expect("thread trace dump instructions 2", substrs=["main.cpp:11"])

        # We stop tracing all
        self.runCmd("thread trace stop all")

        # The trace is still in memory
        self.expect("thread trace dump instructions 2", substrs=["main.cpp:11"])

        # We'll stop at the next breakpoint in thread 3, and nothing should be traced
        self.expect("continue")
        self.expect(
            "thread trace dump instructions 3", substrs=["not traced"], error=True
        )
        self.expect(
            "thread trace dump instructions 1", substrs=["not traced"], error=True
        )
        self.expect(
            "thread trace dump instructions 2", substrs=["not traced"], error=True
        )

    @skipIf(oslist=no_match(["linux"]), archs=no_match(["i386", "x86_64"]))
    @testSBAPIAndCommands
    def testStartMultipleLiveThreadsWithSmallTotalLimit(self):
        self.build()
        exe = self.getBuildArtifact("a.out")

        self.dbg.CreateTarget(exe)

        self.expect("b main")
        self.expect("r")

        # trace the entire process with enough total size for 1 thread trace
        self.traceStartProcess(processBufferSizeLimit=5000)

        # we get the stop event when trace 2 appears and can't be traced
        self.expect("c", substrs=["Thread", "can't be traced"])
        # we get the stop event when trace 3 appears and can't be traced
        self.expect("c", substrs=["Thread", "can't be traced"])

        self.traceStopProcess()

    @skipIf(oslist=no_match(["linux"]), archs=no_match(["i386", "x86_64"]))
    @testSBAPIAndCommands
    def testStartPerCpuSession(self):
        self.skipIfPerCpuTracingIsNotSupported()

        self.build()
        exe = self.getBuildArtifact("a.out")
        self.dbg.CreateTarget(exe)

        self.expect("b main")
        self.expect("r")

        # We should fail if we hit the total buffer limit. Useful if the number
        # of cpus is huge.
        self.traceStartProcess(
            error="True",
            processBufferSizeLimit=100,
            perCpuTracing=True,
            substrs=[
                "The process can't be traced because the process trace size "
                "limit has been reached. Consider retracing with a higher limit."
            ],
        )

        self.traceStartProcess(perCpuTracing=True)
        self.traceStopProcess()

        self.traceStartProcess(perCpuTracing=True)
        # We can't support multiple per-cpu tracing sessions.
        self.traceStartProcess(
            error=True,
            perCpuTracing=True,
            substrs=["Process currently traced. Stop process tracing first"],
        )

        # We can't support tracing per thread is per cpu is enabled.
        self.traceStartThread(
            error="True", substrs=["Thread with tid ", "is currently traced"]
        )

        # We can't stop individual thread when per cpu is enabled.
        self.traceStopThread(
            error="True",
            substrs=[
                "Can't stop tracing an individual thread when per-cpu process tracing is enabled"
            ],
        )

        # We move forward a little bit to collect some data
        self.expect("b 19")
        self.expect("c")

        # We will assert that the trace state will contain valid context switch and intel pt trace buffer entries.
        # Besides that, we need to get tsc-to-nanos conversion information.

        # We first parse the json response from the custom packet
        self.runCmd(
            """process plugin packet send 'jLLDBTraceGetState:{"type":"intel-pt"}]'"""
        )
        response_header = "response: "
        output = None
        for line in self.res.GetOutput().splitlines():
            if line.find(response_header) != -1:
                response = line[
                    line.find(response_header) + len(response_header) :
                ].strip()
                output = json.loads(response)

        self.assertIsNotNone(output)
        self.assertIn("cpus", output)
        self.assertIn("tscPerfZeroConversion", output)
        found_non_empty_context_switch = False

        for cpu in output["cpus"]:
            context_switch_size = None
            ipt_trace_size = None
            for binary_data in cpu["binaryData"]:
                if binary_data["kind"] == "iptTrace":
                    ipt_trace_size = binary_data["size"]
                elif binary_data["kind"] == "perfContextSwitchTrace":
                    context_switch_size = binary_data["size"]
            self.assertIsNotNone(context_switch_size)
            self.assertIsNotNone(ipt_trace_size)
            if context_switch_size > 0:
                found_non_empty_context_switch = True

        # We must have captured the context switch of when the target resumed
        self.assertTrue(found_non_empty_context_switch)

        self.expect("thread trace dump instructions")

        self.traceStopProcess()



#!/usr/bin/env python

# ----------------------------------------------------------------------
# For the shells csh, tcsh:
#   ( setenv PYTHONPATH /Applications/Xcode.app/Contents/SharedFrameworks/LLDB.framework/Resources/Python ; ./globals.py <path> [<path> ...])
#
# For the shells sh, bash:
#   PYTHONPATH=/Applications/Xcode.app/Contents/SharedFrameworks/LLDB.framework/Resources/Python ./globals.py <path> [<path> ...]
# ----------------------------------------------------------------------

import lldb
import optparse
import os
import shlex
import sys


def get_globals(raw_path, options):
    error = lldb.SBError()
    # Resolve the path if needed
    path = os.path.expanduser(raw_path)
    # Create a target using path + options
    target = lldb.debugger.CreateTarget(
        path, options.arch, options.platform, False, error
    )
    if target:
        # Get the executable module
        module = target.module[target.executable.basename]
        if module:
            # Keep track of which variables we have already looked up
            global_names = list()
            # Iterate through all symbols in the symbol table and watch for any
            # DATA symbols
            for symbol in module.symbols:
                if symbol.type == lldb.eSymbolTypeData:
                    # The symbol is a DATA symbol, lets try and find all global variables
                    # that match this name and print them
                    global_name = symbol.name
                    # Make sure we don't lookup the same variable twice
                    if global_name not in global_names:
                        global_names.append(global_name)
                        # Find all global variables by name
                        global_variable_list = module.FindGlobalVariables(
                            target, global_name, lldb.UINT32_MAX
                        )
                        if global_variable_list:
                            # Print results for anything that matched
                            for global_variable in global_variable_list:
                                # returns the global variable name as a string
                                print("name = %s" % global_variable.name)
                                # Returns the variable value as a string
                                print("value = %s" % global_variable.value)
                                print(
                                    "type = %s" % global_variable.type
                                )  # Returns an lldb.SBType object
                                # Returns an lldb.SBAddress (section offset
                                # address) for this global
                                print("addr = %s" % global_variable.addr)
                                # Returns the file virtual address for this
                                # global
                                print(
                                    "file_addr = 0x%x" % global_variable.addr.file_addr
                                )
                                # returns the global variable value as a string
                                print("location = %s" % global_variable.location)
                                # Returns the size in bytes of this global
                                # variable
                                print("size = %s" % global_variable.size)
                                print()


def globals(command_args):
    """Extract all globals from any arguments which must be paths to object files."""
    usage = "usage: %prog [options] <PATH> [PATH ...]"
    description = """This command will find all globals in the specified object file and return an list() of lldb.SBValue objects (which might be empty)."""
    parser = optparse.OptionParser(description=description, prog="globals", usage=usage)
    parser.add_option(
        "-v",
        "--verbose",
        action="store_true",
        dest="verbose",
        help="display verbose debug info",
        default=False,
    )
    parser.add_option(
        "-a",
        "--arch",
        type="string",
        metavar="arch",
        dest="arch",
        help="Specify an architecture (or triple) to use when extracting from a file.",
    )
    parser.add_option(
        "-p",
        "--platform",
        type="string",
        metavar="platform",
        dest="platform",
        help='Specify the platform to use when creating the debug target. Valid values include "localhost", "darwin-kernel", "ios-simulator", "remote-freebsd", "remote-macosx", "remote-ios", "remote-linux".',
    )
    try:
        (options, args) = parser.parse_args(command_args)
    except:
        return

    for path in args:
        get_globals(path, options)


if __name__ == "__main__":
    lldb.debugger = lldb.SBDebugger.Create()
    globals(sys.argv[1:])



"""
Test finish out of an empty function (may be one-instruction long)
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class FinishFromEmptyFunctionTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipIf(compiler="clang", compiler_version=['<', '17.0'])
    def test_finish_from_empty_function(self):
        """Test that when stopped at a breakpoint located at the last instruction
        of a function, finish leaves it correctly."""
        self.build()
        target, _, thread, _ = lldbutil.run_to_source_breakpoint(
            self, "// Set breakpoint here", lldb.SBFileSpec("main.c")
        )
        # Find the address of the last instruction of 'done()' and set a breakpoint there.
        # Even though 'done()' is empty, it may contain prologue and epilogue code, so
        # simply setting a breakpoint at the function can place it before 'ret'.
        error = lldb.SBError()
        ret_bp_addr = lldb.SBAddress()
        while True:
            thread.StepInstruction(False, error)
            self.assertTrue(error.Success())
            frame = thread.GetSelectedFrame()
            if "done" in frame.GetFunctionName():
                ret_bp_addr = frame.GetPCAddress()
            elif ret_bp_addr.IsValid():
                # The entire function 'done()' has been stepped through, so 'ret_bp_addr'
                # now contains the address of its last instruction, i.e. 'ret'.
                break
        ret_bp = target.BreakpointCreateByAddress(ret_bp_addr.GetLoadAddress(target))
        self.assertTrue(ret_bp.IsValid())
        # Resume the execution and hit the new breakpoint.
        self.runCmd("cont")
        if self.TraceOn():
            self.runCmd("bt")

        correct_stepped_out_line = line_number("main.c", "leaving main")
        return_statement_line = line_number("main.c", "return 0")
        safety_bp = target.BreakpointCreateByLocation(
            lldb.SBFileSpec("main.c"), return_statement_line
        )
        self.assertTrue(safety_bp.IsValid())

        thread.StepOut(error)
        self.assertTrue(error.Success())

        if self.TraceOn():
            self.runCmd("bt")

        frame = thread.GetSelectedFrame()
        self.assertEqual(
            frame.line_entry.GetLine(),
            correct_stepped_out_line,
            "Step-out lost control of execution, ran too far",
        )



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *


class GuiTestCase(TestBase):
    @no_debug_info_test
    @skipIfCursesSupportMissing
    def test_reproducer_generate_invalid_invocation(self):
        self.expect(
            "gui blub", error=True, substrs=["'gui' doesn't take any arguments."]
        )



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class SyntheticOneLineSummariesTestCase(TestBase):
    def test(self):
        """Test that the presence of a synthetic child provider doesn't prevent one-line-summaries."""
        self.build()
        lldbutil.run_to_source_breakpoint(self, "break here", lldb.SBFileSpec("main.c"))

        # set up the synthetic children provider
        self.runCmd("script from MyStringFormatter import *")
        self.runCmd("type synth add -l MyStringSynthProvider MyString")
        self.runCmd('type summary add --summary-string "${var.guts}" MyString')

        self.expect(
            "frame variable s",
            substrs=['a = "hello", b = "world"'],
        )
